﻿using LiquidacionGastos.UnitsOfWork;
using LiquidacionGastos.Models;
using LiquidacionGastos.Repositories;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;
using System.Transactions;
using System.Diagnostics;

namespace LiquidacionGastos.Business
{
    public class LiquidacionManagement //: IDisposable
    {
        //private static object contextCountLock = new object();
        //private static int ContextCount = 0;
        //private static LGContext _Context;
        //private LGContext Context { get { return _Context; } set { _Context = value; } }      

        /// <summary>
        /// Constructor por defecto.
        /// </summary>
        public LiquidacionManagement()
        {
            //lock (contextCountLock)
            //{
            //    if (ContextCount == 0)
            //        this.Context = new LGContext();

            //    ContextCount++;
            //    Debug.WriteLine(string.Format("Abriendo contexto para LiquidacionManagement. Número de referencias al contexto: {0}", ContextCount));
            //}
        }

        //public void Dispose()
        //{
        //    lock (contextCountLock)
        //    {   
        //        ContextCount--;

        //        if (ContextCount == 0)
        //            this.Context.Dispose();

        //        Debug.WriteLine(string.Format("Cerrando contexto para LiquidacionManagement. Número de referencias al contexto: {0}", ContextCount));
        //    }
        //}

        //~LiquidacionManagement()
        //{
        //    this.Dispose();
        //}

        #region [Gestión de liquidaciones]

        ///// <summary>
        ///// Crear un nuevo objeto de tipo Liquidacion.
        ///// </summary>
        ///// <returns>El nuevo objeto de tipo Usuario</returns>
        public Liquidacion GetNewLiquidacion(Usuario usuario)
        {
            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                return uow.GetNewLiquidacion(usuario);
            }
        }

        public CambioEstadoLiquidacion ChangeEstadoLiquidacion(Liquidacion liquidacion, EstadoLiquidacionType estadoNuevo)
        {
            CambioEstadoLiquidacion cambioEstado = null;

            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                cambioEstado = uow.ChangeEstadoLiquidacion(liquidacion, estadoNuevo, string.Empty);
                uow.Commit();
            }

            return cambioEstado;
        }

        public CambioEstadoLiquidacion ChangeEstadoLiquidacion(Liquidacion liquidacion, EstadoLiquidacionType estadoNuevo, string motivoRechazo)
        {
            CambioEstadoLiquidacion cambioEstado = null;

            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                cambioEstado = uow.ChangeEstadoLiquidacion(liquidacion, estadoNuevo, motivoRechazo);
                uow.Commit();
            }

            return cambioEstado;
        }

        public Liquidacion GetLiquidacion(Guid idLiquidacion)
        {
            if (idLiquidacion == null)
                throw new ArgumentNullException("idLiquidacion");

            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                return uow.GetLiquidacion(idLiquidacion);
            }
        }

        public void InsertLiquidacion(Liquidacion liquidacion)
        {
            if (liquidacion == null)
                throw new ArgumentNullException("liquidacion");

            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                uow.InsertLiquidacion(liquidacion);
                uow.Commit();
            }
        }

        public void UpdateLiquidacion(Liquidacion liquidacion)
        {
            if (liquidacion == null)
                throw new ArgumentNullException("liquidacion");

            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                uow.UpdateLiquidacion(liquidacion);
                uow.Commit();
            }
        }

        public void DeleteLiquidacion(Liquidacion liquidacion)
        {
            if (liquidacion == null)
                throw new ArgumentNullException("liquidacion");

            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                uow.DeleteLiquidacion(liquidacion);
                uow.Commit();
            }
        }

        public void UpdateImporte(Liquidacion liquidacion)
        {
            if (liquidacion == null)
                throw new ArgumentNullException("liquidacion");

            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                uow.UpdateImporte(liquidacion);
            }
        }

        public ICollection<Liquidacion> ListLiquidacionesByUsuario(Usuario usuario, params EstadoLiquidacionType[] estadosLiquidacion)
        {
            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                // Si no existe ninguna liquidación abierta, crea una nueva.
                uow.CheckLiquidacionAbierta(usuario);

                // Guardar la nueva liquidación creada.
                uow.Commit();

                return uow.ListLiquidacionesByUsuario(usuario, estadosLiquidacion);
            }
        }

        /// <summary>
        /// Devuelve la lista de liquidaciones en las que el usuario es autorizador.
        /// </summary>
        /// <param name="autorizador"></param>
        /// <returns>La colección de </returns>
        public ICollection<Liquidacion> ListLiquidacionesByAutorizador(Usuario autorizador, params EstadoLiquidacionType[] estadosLiquidacion)
        {
            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                return uow.ListLiquidacionesByAutorizador(autorizador, estadosLiquidacion);
            }
        }

        #endregion [Gestión de liquidaciones]

        #region [Gestión de gastos]

        public Gasto GetNewGasto(Liquidacion liquidacion)
        {
            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                return uow.GetNewGasto(liquidacion);
            }
        }

        public Gasto GetGasto(Guid idGasto)
        {
            if (idGasto == null)
                throw new ArgumentNullException("idGasto");

            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                return uow.GetGasto(idGasto);
            }
        }

        public void InsertGasto(Gasto gasto)
        {
            if (gasto == null)
                throw new ArgumentNullException("gasto");

            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                uow.InsertGasto(gasto);                
                uow.Commit();
            }
        }

        public void UpdateGasto(Gasto gasto)
        {
            if (gasto == null)
                throw new ArgumentNullException("gasto");

            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                uow.UpdateGasto(gasto);
                uow.Commit();
            }
        }

        public void DeleteGasto(Gasto gasto)
        {
            if (gasto == null)
                throw new ArgumentNullException("gasto");

            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                uow.DeleteGasto(gasto);
                uow.Commit();
            }
        }

        public void RefreshGasto(Gasto gasto)
        {
            if (gasto == null)
                throw new ArgumentNullException("gasto");

            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                uow.RefreshGasto(gasto);
            }
        }

        public ICollection<Gasto> ListGastos()
        {
            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                return uow.ListGastos();
            }
        }

        public ICollection<Gasto> ListGastosByLiquidacion(Liquidacion liquidacion)
        {
            if (liquidacion == null)
                throw new ArgumentNullException("liquidacion");

            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                return uow.ListGastosByLiquidacion(liquidacion);
            }
        }

        #endregion [Gestión de gastos]

        #region [Gestión de tipos de gastos]

        public ICollection<TipoGasto> ListTiposGasto()
        {
            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                return uow.ListTiposGasto();
            }
        }

        public ICollection<CambioMoneda> ListCambiosMonedaMostRecent()
        {
            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                return uow.ListCambiosMonedaMostRecent();
            }
        }

        #endregion [Gestión de tipos de gastos]
    }
}
