﻿using LiquidacionGastos.UnitsOfWork;
using LiquidacionGastos.Models;
using LiquidacionGastos.Repositories;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;
using System.Transactions;

namespace LiquidacionGastos.Business
{
    public class OrganizacionManagement //: IDisposable
    {
        //private static object contextCountLock = new object();
        //private static int ContextCount = 0;
        //private static LGContext _Context;
        //private LGContext Context { get { return _Context; } set { _Context = value; } }      

        /// <summary>
        /// Constructor por defecto.
        /// </summary>
        public OrganizacionManagement() 
        {
            //lock (contextCountLock)
            //{
            //    if (ContextCount == 0)
            //        this.Context = new LGContext();

            //    ContextCount++;
            //    Debug.WriteLine(string.Format("Abriendo contexto para OrganizacionManagement. Número de referencias al contexto: {0}", ContextCount));
            //}
        }

        //public void Dispose()
        //{
        //    lock (contextCountLock)
        //    {   
        //        ContextCount--;

        //        if (ContextCount == 0)
        //            this.Context.Dispose();

        //        Debug.WriteLine(string.Format("Cerrando contexto para OrganizacionManagement. Número de referencias al contexto: {0}", ContextCount));
        //    }
        //}
                
        //~OrganizacionManagement()
        //{
        //    this.Dispose();
        //}

        #region [Gestión de usuarios]

        /// <summary>
        /// Crear un nuevo objeto de tipo Usuario.
        /// </summary>
        /// <returns>El nuevo objeto de tipo Usuario</returns>
        public Usuario GetNewUsuario()
        {
            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.GetNewUsuario();
            }
        }

        /// <summary>
        /// Devuelve un usuario determinado por el valor de su propiedad identificador.
        /// No confundir la propiedad identificador con IdUsuario.
        /// </summary>
        /// <param name="identificador"></param>
        public Usuario GetUsuario(string identificador)
        {
            Debug.WriteLine(string.Format("Cliente: Solicitiud de usuario con identificador {0}", identificador));

            if (string.IsNullOrEmpty(identificador))
                throw new ArgumentNullException("identificador");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.GetUsuarioByIdentificador(identificador);
            }
        }

        public Usuario GetUsuario(Guid idUsuario)
        {
            if (idUsuario == null)
                throw new ArgumentNullException("idUsuario");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.GetUsuario(idUsuario);
            }
        }

        public void InsertUsuario(Usuario usuario)
        {
            if (usuario == null)
                throw new ArgumentNullException("usuario");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                uow.InsertUsuario(usuario);
                uow.Commit();
            }
        }

        public void UpdateUsuario(Usuario usuario)
        {
            if (usuario == null)
                throw new ArgumentNullException("usuario");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                uow.UpdateUsuario(usuario);
                uow.Commit();
            }
        }

        public void DeleteUsuario(Usuario usuario)
        {
            if (usuario == null)
                throw new ArgumentNullException("usuario");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                uow.DeleteUsuario(usuario);
                uow.Commit();
            }
        }

        public void RefreshUsuario(Usuario usuario)
        {
            if (usuario == null)
                throw new ArgumentNullException("usuario");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                uow.RefreshUsuario(usuario);
            }
        }

        public static void SetPassword(Usuario usuario, string password)
        {
            if (usuario == null)
                throw new ArgumentNullException("usuario");

            usuario.HashContrasena = Security.PasswordHash.CreateHash(password);
        }

        public ICollection<Usuario> ListUsuarios()
        {

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.ListUsuarios();
            }
        }

        public ICollection<Usuario> ListUsuariosByNombreUsuario(string nombreUsuario)
        {
            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.ListUsuariosByNombreUsuario(nombreUsuario);
            }
        }

        public ICollection<Usuario> ListUsuariosByNombreDepartamento(string nombreDepartamento)
        {
            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.ListUsuariosByNombreDepartamento(nombreDepartamento);
            }
        }

        #endregion [Gestión de usuarios]

        #region [Gestión de departamentos]

        public Departamento GetNewDepartamento()
        {
            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.GetNewDepartamento();
            }
        }

        public Departamento GetDepartamento(Guid idDepartamento)
        {
            if (idDepartamento == null)
                throw new ArgumentNullException("idDepartamento");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.GetDepartamento(idDepartamento);
            }
        }

        public void InsertDepartamento(Departamento departamento)
        {
            if (departamento == null)
                throw new ArgumentNullException("departamento");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                uow.InsertDepartamento(departamento);
                uow.Commit();
            }
        }

        public void UpdateDepartamento(Departamento departamento)
        {
            if (departamento == null)
                throw new ArgumentNullException("departamento");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                uow.UpdateDepartamento(departamento);
                uow.Commit();
            }
        }

        public void DeleteDepartamento(Departamento departamento)
        {
            if (departamento == null)
                throw new ArgumentNullException("departamento");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                uow.DeleteDepartamento(departamento);
                uow.Commit();
            }

        }

        public void RefreshDepartamento(Departamento departamento)
        {
            if (departamento == null)
                throw new ArgumentNullException("departamento");
            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                uow.RefreshDepartamento(departamento);
            }
        }

        public ICollection<Departamento> ListDepartamentos()
        {
            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.ListDepartamentos();
            }
        }

        public ICollection<Departamento> ListDepartamentosByNombreDepartamento(string nombreDepartamento)
        {
            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.ListDepartamentosByNombreDepartamento(nombreDepartamento);
            }
        }

        public ICollection<Departamento> ListDepartamentosBySuperdepartamento(Departamento superdepartamento)
        {
            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.ListDepartamentosBySuperdepartamento(superdepartamento);
            }
        }

        #endregion [Gestión de departamentos]

        #region [Gestión de calendarios]

        public Calendario GetNewCalendario()
        {
            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.GetNewCalendario();
            }
        }

        public void InsertCalendario(Calendario calendario)
        {
            if (calendario == null)
                throw new ArgumentNullException("calendario");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                uow.InsertCalendario(calendario);
                uow.Commit();
            }
        }

        public void UpdateCalendario(Calendario calendario)
        {
            if (calendario == null)
                throw new ArgumentNullException("calendario");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                uow.UpdateCalendario(calendario);
                uow.Commit();
            }
        }

        public void DeleteCalendario(Calendario calendario)
        {
            if (calendario == null)
                throw new ArgumentNullException("calendario");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                if (calendario.Liquidaciones.Count() > 0)
                    throw new BusinessRuleException(string.Format("El calendario {0} no puede ser eliminado porque ya ha sido utilizado en procesos de liquidación", calendario.Nombre));

                uow.DeleteCalendario(calendario);
                uow.Commit();
            }
        }

        public void RefreshCalendario(Calendario calendario)
        {
            if (calendario == null)
                throw new ArgumentNullException("calendario");

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                uow.RefreshCalendario(calendario);
            }
        }

        public ICollection<Calendario> ListCalendarios()
        {
            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.ListCalendarios();
            }
        }

        public ICollection<Calendario> ListCalendariosByNombreCalendario(string nombreCalendario)
        {
            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                return uow.ListCalendariosByNombreCalendario(nombreCalendario);
            }
        }

        public Calendario DuplicateCalendario(Calendario calendario)
        {
            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                Calendario newCalendario = uow.GetNewCalendario();

                newCalendario.Nombre = "Copia de " + calendario.Nombre;
                newCalendario.DiasFestivos = calendario.DiasFestivos;

                uow.InsertCalendario(newCalendario);
                uow.Commit();
                return newCalendario;
            }
        }

        public void ResetCalendario(Calendario calendario)
        {
            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                calendario.DiasFestivos = new byte[calendario.DiasFestivos.Length];
                uow.UpdateCalendario(calendario);
                uow.Commit();
            }

        }

        #endregion [Gestión de calendarios]
    }
}

