﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.Framework
{
    public class ServiceLocator 
    {
        private static readonly IDictionary<Type, IDictionary<string, object>> _Services = new Dictionary<Type, IDictionary<string, object>>();

        private static object GetServiceInstance(Type serviceType, string key)
        {
            // Si no existe el servicio devolver.
            if (!_Services.ContainsKey(serviceType))
                return null;
                //throw new ServiceNotFoundException(string.Format("Service of type {0} was not found.", serviceType.Name));
                

            // Si no existe la instancia solicitada devolver null.
            if (!_Services[serviceType].ContainsKey(key))
                return null;
                //throw new ServiceNotFoundException(string.Format("Service instance of type {0} for key {1} was not found.", serviceType.Name, key));

            return _Services[serviceType][key];
        }

        private static void RegisterServiceInstance(Type serviceType, string key, object serviceInstance)
        {
            if (!_Services.ContainsKey(serviceType))
                _Services.Add(serviceType, new Dictionary<string, object>());

            if (_Services[serviceType].ContainsKey(key))
                throw new ServiceLocationException(string.Format("Service instance of type {0} for key {1} is already registered and is not possible to be registered again.", serviceType.Name, key));

            _Services[serviceType].Add(key, serviceInstance);
        }

        private static void UnregisterServiceInstance(Type serviceType, string key)
        {
            if (!_Services.ContainsKey(serviceType))
                throw new ServiceNotFoundException(string.Format("Service of type {0} was not found.", serviceType.Name));

            if (!_Services[serviceType].ContainsKey(key))
                throw new ServiceNotFoundException(string.Format("Service instance of type {0} for key {1} was not found.", serviceType.Name, key));

            _Services[serviceType].Remove(key);

            if (_Services[serviceType].Count() == 0)
                _Services.Remove(serviceType);
        }
        
        public static TService Resolve<TService>(string key)
        {
            return (TService)GetServiceInstance(typeof(TService), key);
        }

        public static TService Resolve<TService>()
        {
            return Resolve<TService>(string.Empty);
        }

        public static void Register<TService>(string key, object serviceInstance)
        {
            RegisterServiceInstance(typeof(TService), key, serviceInstance);
        }

        public static void Register<TService>(object serviceInstance)
        {
            Register<TService>(string.Empty, serviceInstance);
        }

        public static void Unregister<TService>(string key)
        {
            UnregisterServiceInstance(typeof(TService), key);
        }

        public static void Unregister<TService>()
        {
            Unregister<TService>(string.Empty);
        }
    }
}
