using LiquidacionGastos.Framework;
using System;
using System.Collections.Generic;

namespace LiquidacionGastos.Models
{
    public partial class Liquidacion : ObservableObject
    {
        public Liquidacion()
        {
            this.CambiosEstado = new List<CambioEstadoLiquidacion>();
            this.Gastos = new List<Gasto>();
        }

        // Identificador de la liquidacin.
        private System.Guid _IdLiquidacion;
        public System.Guid IdLiquidacion { get { return _IdLiquidacion; } set { _IdLiquidacion = value; OnPropertyChanged(); } }

        // Fecha (Mes y Ao, el da es 1) del periodo de la hoja de liquidacin.
        private System.DateTime _Fecha;
        public System.DateTime Fecha { get { return _Fecha; } set { _Fecha = value; OnPropertyChanged(); } }

        // Importe total de los gastos incluidos en la liquidacion.
        private decimal _Importe;
        public decimal Importe { get { return _Importe; } set { _Importe = value; OnPropertyChanged(); } }

        // Descripcin de la causa en caso de rechazo.
        private string _MotivoRechazo;
        public string MotivoRechazo { get { return _MotivoRechazo; } set { _MotivoRechazo = value; OnPropertyChanged(); } }
                        
        // Cambios de estado que se han producido en esta liquidacin.
        private ICollection<CambioEstadoLiquidacion> _CambiosEstado;
        public ICollection<CambioEstadoLiquidacion> CambiosEstado { get { return _CambiosEstado; } set { _CambiosEstado = value; OnPropertyChanged(); } }

        // Estado de la liquidacin actual.
        private int _IdEstado;
        public int IdEstado { get { return _IdEstado; } set { _IdEstado = value; OnPropertyChanged(); } }        
        private EstadoLiquidacion _Estado;
        public EstadoLiquidacion Estado
        {
            get { return _Estado; }
            set
            {
                _Estado = value;

                if (_Estado != null)
                    _IdEstado = _Estado.IdEstadoLiquidacion;

                OnPropertyChanged();
            }
        }        


        // Propiedad calculada. NO se almacena en base de datos.
        public bool EstaAutorizada
        { 
            get { 
                if (Estado == null)
                    return false;

                return Estado.IdEstadoLiquidacion == (int)Models.EstadoLiquidacionType.Autorizada;
            }
        }

        // Gastos que contiene esta hoja de liquidacin.
        private ICollection<Gasto> _Gastos;
        public ICollection<Gasto> Gastos { get { return _Gastos; } set { _Gastos = value; OnPropertyChanged(); } }

        // Usuario propietario de la hoja de liquidacin.
        private System.Guid _IdUsuario;
        public System.Guid IdUsuario { get { return _IdUsuario; } set { _IdUsuario = value; OnPropertyChanged(); } }
        private Usuario _Usuario;
        public Usuario Usuario
        {
            get { return _Usuario; }
            set
            {
                _Usuario = value;

                if (_Usuario != null)
                    _IdUsuario = _Usuario.IdUsuario;

                OnPropertyChanged();
            }
        }

        // Calendario utilizado en el computo de dietas.
        private Nullable<System.Guid> _IdCalendario;
        public Nullable<System.Guid> IdCalendario { get { return _IdCalendario; } set { _IdCalendario = value; OnPropertyChanged(); } }
        private Calendario _Calendario;
        public Calendario Calendario
        {
            get { return _Calendario; }
            set
            {
                _Calendario = value;

                if (_Calendario != null)
                    _IdCalendario = _Calendario.IdCalendario;

                OnPropertyChanged();
            }
        }
        
    }
}
