using System.ComponentModel.DataAnnotations.Schema;
using System.Data.Entity.ModelConfiguration;

namespace LiquidacionGastos.Models.Mapping
{
    public class GastoMap : EntityTypeConfiguration<Gasto>
    {
        public GastoMap()
        {
            // Primary Key
            this.HasKey(t => t.IdGasto);

            // Properties
            this.Property(t => t.Descripcion)
                .IsRequired()
                .HasMaxLength(255);
            this.Property(t => t.ImagenComprobante)
                .HasColumnType("varbinary(MAX)");

            // Table & Column Mappings
            this.ToTable("Gastos");
            this.Property(t => t.IdGasto).HasColumnName("IdGasto");
            this.Property(t => t.Fecha).HasColumnName("FechaGasto");
            this.Property(t => t.Importe).HasColumnName("ImporteGasto");
            this.Property(t => t.Descripcion).HasColumnName("DescripcionGasto");
            this.Property(t => t.IdTipo).HasColumnName("IdTipoGasto");
            this.Property(t => t.IdLiquidacion).HasColumnName("IdLiquidacion");
            this.Property(t => t.IdCambioMoneda).HasColumnName("IdCambioMoneda");
            this.Property(t => t.ImagenComprobante).HasColumnName("ImagenComprobante");

            // Relationships
            this.HasRequired(t => t.Liquidacion)
                .WithMany(t => t.Gastos)
                .HasForeignKey(d => d.IdLiquidacion);
            this.HasRequired(t => t.Tipo)
                .WithMany(t => t.Gastos)
                .HasForeignKey(d => d.IdTipo);
            this.HasRequired(t => t.CambioMoneda)
                .WithMany(t => t.Gastos)
                .HasForeignKey(d => d.IdCambioMoneda);
        }
    }
}
