using LiquidacionGastos.Framework;
using System;
using System.Collections.Generic;

namespace LiquidacionGastos.Models
{
    public partial class Usuario : ObservableObject
    {
        public Usuario()
        {
            this.DepartamentosSupervisados = new List<Departamento>();
            this.Liquidaciones = new List<Liquidacion>();
            this.Parametros = new List<ParametroSistema>();
        }        
       
        // Identificador del usuario.
        private System.Guid _IdUsuario;
        public System.Guid IdUsuario { get { return _IdUsuario; } set { _IdUsuario = value; OnPropertyChanged(); } }

        // Nombre completo del usuario.        
        private string _Nombre;
        public string Nombre { get { return _Nombre; } set { _Nombre = value; OnPropertyChanged(); } }

        // Identificador de usuario para el proceso de login.
        private string _Identificador;
        public string Identificador { get { return _Identificador; } set { _Identificador = value; OnPropertyChanged(); } }

        // Hash de la contrasea del usuario
        private string _HashContrasena;
        public string HashContrasena { get { return _HashContrasena; } set { _HashContrasena = value; OnPropertyChanged(); } }

        // Indicador de usuario habilitado para realizar login o no.
        private bool _EstaActivadoLogin;
        public bool EstaActivadoLogin { get { return _EstaActivadoLogin; } set { _EstaActivadoLogin = value; OnPropertyChanged(); } }

        // Direccin de email del usuario.
        private string _Email;
        public string Email { get { return _Email; } set { _Email = value; OnPropertyChanged(); } }

        // Indicador de notificaciones al usuario va mail.
        private bool _EstaActivadoNotificarPorEmail;
        public bool EstaActivadoNotificarPorEmail { get { return _EstaActivadoNotificarPorEmail; } set { _EstaActivadoNotificarPorEmail = value; OnPropertyChanged(); } }

        // Indicador de perfil administrador.
        private bool _EsAdministrador;
        public bool EsAdministrador { get { return _EsAdministrador; } set { _EsAdministrador = value; OnPropertyChanged(); } }
        
        // Departamentos en los que el usuario es reponsable.
        private ICollection<Departamento> _DepartamentosSupervisados;
        public ICollection<Departamento> DepartamentosSupervisados { get { return _DepartamentosSupervisados; } set { _DepartamentosSupervisados = value; OnPropertyChanged(); } }

        // Hojas de liquidacin del usuario.
        private ICollection<Liquidacion> _Liquidaciones;
        public ICollection<Liquidacion> Liquidaciones { get { return _Liquidaciones; } set { _Liquidaciones = value; OnPropertyChanged(); } }

        // Departamento al que pertenece el usuario.
        private Nullable<System.Guid> _IdDepartamento;
        public Nullable<System.Guid> IdDepartamento { get { return _IdDepartamento; } set { _IdDepartamento = value; OnPropertyChanged(); } }
        private Departamento _Departamento;
        public Departamento Departamento 
        { 
            get { return _Departamento; } 
            set 
            { 
                _Departamento = value;
                
                if (_Departamento != null)
                    _IdDepartamento = _Departamento.IdDepartamento;

                OnPropertyChanged(); 
            } 
        }

        // Calendario utilizado en el computo de dietas por el usuario.
        //private Nullable<System.Guid> _IdCalendario;
        //public Nullable<System.Guid> IdCalendario { get { return _IdCalendario; } set { _IdCalendario = value; OnPropertyChanged(); } }
        //private Calendario _Calendario;
        //public Calendario Calendario
        //{
        //    get { return _Calendario; }
        //    set
        //    {
        //        _Calendario = value;

        //        if (_Calendario != null)
        //            _IdCalendario = _Calendario.IdCalendario;

        //        OnPropertyChanged();
        //    }
        //}

        // Parametros asociados al usuario.
        public ICollection<ParametroSistema> Parametros { get; set; }
    }
}
