﻿using LiquidacionGastos.Models;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.Repositories
{
    public class CalendarioRepository : EntityFrameworkTrackedRepository<Calendario>
    {
        public CalendarioRepository(LGContext context) : base(context) 
        {
        }

        protected override Nullable<Guid> GetGuid(Calendario entity)
        {
            return entity.IdCalendario;
        }

        public Calendario CreateCalendario()
        {
            Calendario calendario = new Calendario()
            {
                IdCalendario = Guid.NewGuid()
            };

            return calendario;
        }

        public Calendario GetCalendario(Guid idCalendario)
        {
            Calendario calendario = this.Get(d => d.IdCalendario == idCalendario);

            //if (calendario == null)
            //    throw new DataNotFoundException("idCalendario", idCalendario, "No existe ningún calendario con el valor de idCalendario especificado.");

            return calendario;
        }

        public Calendario GetCalendarioByNombre(string nombre)
        {
            Calendario calendario = this.Get(c => c.Nombre == nombre);

            if (calendario == null)
                throw new DataNotFoundException("calendario", calendario, "No existe ningún calendario con el nombre especificado.");

            return calendario;
        }

        public IQueryable<Calendario> ListCalendarios()
        {
            return this.ListCalendarios(null);
        }

        public IQueryable<Calendario> ListCalendarios(Expression<Func<Calendario, bool>> filterPredicate)
        {
            if (filterPredicate == null)
                filterPredicate = x => true;

            return this.List(filterPredicate);//.Include();
        }

        public void InsertCalendario(Calendario calendario)
        {
            this.Insert(calendario);
        }

        public void UpdateCalendario(Calendario calendario)
        {
            this.Update(calendario);
        }

        public void DeleteCalendario(Calendario calendario)
        {
            this.Delete(calendario);
        }

        public void RefreshCalendario(Calendario calendario)
        {
            this.Refresh(calendario);
        }
    }
}
