﻿using LiquidacionGastos.Models;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.Repositories
{
    public class CambioMonedaRepository : EntityFrameworkTrackedRepository<CambioMoneda>
    {
        //public CambioMonedaRepository()
        //    : base()
        //{
        //}

        public CambioMonedaRepository(LGContext context)
            : base(context) 
        {
        }

        protected override Nullable<Guid> GetGuid(CambioMoneda entity)
        {
            return entity.IdCambioMoneda;
        }

        public CambioMoneda CreateCambioMoneda()
        {
            CambioMoneda cambioMoneda = new CambioMoneda()
            {
                IdCambioMoneda = Guid.NewGuid()
            };

            return cambioMoneda;
        }

        public CambioMoneda GetCambioMoneda(Guid idCambioMoneda)
        {
            CambioMoneda cambioMoneda = this.Get(t => t.IdCambioMoneda == idCambioMoneda);

            //if (cambioMoneda == null)
            //    throw new DataNotFoundException("idCambioMoneda", idCambioMoneda, "No existe ningún cambio de moneda con el valor de idCambioMoneda especificado.");

            return cambioMoneda;
        }

        public CambioMoneda GetCambioModenaByCodigo(string codigo, DateTime fechaCambio)
        {
            CambioMoneda cambioMoneda = this.List(t => t.Moneda.Codigo == codigo && t.FechaCambio < fechaCambio)
                                            .OrderBy(t => t.FechaCambio)
                                            .FirstOrDefault();
            if (cambioMoneda == null)
                throw new DataNotFoundException("cambioMoneda", cambioMoneda, "No existe ningún cambio de moneda con el código de moneda especificado.");

            return cambioMoneda;
        }

        public IQueryable<CambioMoneda> ListCambiosMoneda()
        {
            return this.ListCambiosMoneda(null);
        }

        public IQueryable<CambioMoneda> ListCambiosMoneda(Expression<Func<CambioMoneda, bool>> filterPredicate)
        {
            if (filterPredicate == null)
                filterPredicate = x => true;

            return this.List(filterPredicate)
                       .Include(t => t.Moneda);
        }

        public IQueryable<CambioMoneda> ListCambiosMonedaMostRecent()
        {
            var x = from c in this.Context.Set<CambioMoneda>()
                    group c by c.IdMoneda into g
                    select g.OrderByDescending(t => t.FechaCambio).FirstOrDefault();

            return x.Include(t => t.Moneda);
        }

        public void LoadMoneda(CambioMoneda cambioMoneda)
        {
            this.Load(cambioMoneda, t => t.Moneda);
        }

        public void LoadGastos(CambioMoneda cambioMoneda)
        {
            this.Load(cambioMoneda, t => t.Gastos);
        }

        public void InsertCambioMoneda(CambioMoneda cambioMoneda)
        {
            this.Insert(cambioMoneda);
        }

        public void UpdateCambioMoneda(CambioMoneda cambioMoneda)
        {
            this.Update(cambioMoneda);
        }

        public void DeleteCambioMoneda(CambioMoneda cambioMoneda)
        {
            this.Delete(cambioMoneda);
        }

        public void RefreshCambioMoneda(CambioMoneda cambioMoneda)
        {
            this.Refresh(cambioMoneda);
        }

        public class IdMonedaComparer : IEqualityComparer<CambioMoneda>
        {
            public bool Equals(CambioMoneda x, CambioMoneda y)
            {
                return x.IdMoneda == x.IdMoneda;
            }

            public int GetHashCode(CambioMoneda obj)
            {
                return obj.GetHashCode();
            }
        }
    }
}
