﻿using LiquidacionGastos.Models;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.Repositories
{
    public class DepartamentoRepository : EntityFrameworkTrackedRepository<Departamento>
    {
        //public DepartamentoRepository() : base()
        //{
        //}

        public DepartamentoRepository(LGContext context) : base(context) 
        {
        }

        protected override Nullable<Guid> GetGuid(Departamento entity)
        {
            return entity.IdDepartamento;
        }

        public Departamento CreateDepartamento()
        {
            Departamento departamento = new Departamento()
            {
                IdDepartamento = Guid.NewGuid()
            };

            return departamento;
        }

        public Departamento GetDepartamento(Guid idDepartamento)
        {
            Departamento departamento = this.Get(d => d.IdDepartamento == idDepartamento);            

            //if (departamento == null)
            //    throw new DataNotFoundException("idDepartamento", idDepartamento, "No existe ningún departamento con el valor de idDepartamento especificado.");

            return departamento;
        }

        public IQueryable<Departamento> ListDepartamentos()
        {
            return this.ListDepartamentos(null);
        }

        public IQueryable<Departamento> ListDepartamentos(Expression<Func<Departamento, bool>> filterPredicate)
        {
            if (filterPredicate == null)
                filterPredicate = x => true;

            return this.List(filterPredicate)
                       .Include(t => t.Responsable)
                       .Include(t => t.Superdepartamento)
                       .Include(t => t.Subdepartamentos)
                       .Include(t => t.Usuarios);
        }

        public void LoadResponsable(Departamento departamento)
        {
            this.Load(departamento, d => d.Responsable);
        }

        public void InsertDepartamento(Departamento departamento)
        {
            this.Insert(departamento);
        }

        public void UpdateDepartamento(Departamento departamento)
        {
            this.Update(departamento);
        }

        public void DeleteDepartamento(Departamento departamento)
        {
            this.Delete(departamento);
        }

        public void RefreshDepartamento(Departamento departamento)
        {
            this.Refresh(departamento);
        }
    }
}
