﻿using LiquidacionGastos.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.Repositories
{
    public class EntityFrameworkTrackedRepository<TEntity> : EntityFrameworkRepository<TEntity> where TEntity : class
    {
        public bool IsTrackingEnabled { get; set; }

        protected EntityFrameworkTrackedRepository(LGContext context) : base(context)
        {
            this.IsTrackingEnabled = true;
        }

        protected virtual Nullable<Guid> GetGuid(TEntity entity)
        {
            return null;
        }

        protected virtual Nullable<int> GetId(TEntity entity)
        {
            return null;
        }

        private TrackingEntry.EntidadType GetEntidadType(TEntity entity)
        {
            string entityName = entity.GetType().Name;

            switch (entityName)
            {
                case "Calendario":
                    return TrackingEntry.EntidadType.Calendario;

                case "CambioEstadoLiquidacion":
                    return TrackingEntry.EntidadType.CambioEstadoLiquidacion;

                case "CambioMoneda":
                    return TrackingEntry.EntidadType.CambioMoneda;

                case "Departamento":
                    return TrackingEntry.EntidadType.Departamento;

                case "EstadoLiquidacion":
                    return TrackingEntry.EntidadType.EstadoLiquidacion;

                case "Gasto":
                    return TrackingEntry.EntidadType.Gasto;

                case "Liquidacion":
                    return TrackingEntry.EntidadType.Liquidacion;

                case "Moneda":
                    return TrackingEntry.EntidadType.Moneda;

                case "Notificacion":
                    return TrackingEntry.EntidadType.Notificacion;

                case "TipoGasto":
                    return TrackingEntry.EntidadType.TipoGasto;

                case "Usuario":
                    return TrackingEntry.EntidadType.Usuario;
            }

            throw new ArgumentOutOfRangeException(entityName);
        }

        private TrackingEntry CreateTrackingEntry(TEntity entity, TrackingEntry.OperacionType operation)
        {
            TrackingEntry entry = new TrackingEntry();
            entry.IdTracking = Guid.NewGuid();
            entry.Fecha = DateTime.Now;
            entry.Operacion = (int)operation;
            entry.Entidad = (int)this.GetEntidadType(entity);
            entry.Guid = this.GetGuid(entity);
            entry.Id = this.GetId(entity);

            if (entry.Guid == null && entry.Id == null)
                throw new NullReferenceException("No se ha especificdo un identificador para realizar el seguimiento de la entidad");

            return entry;
        }

        protected override void OnAdd(TEntity entity)
        {
            if (!this.IsTrackingEnabled)
                return;

            Context.Set<TrackingEntry>().Add(CreateTrackingEntry(entity, TrackingEntry.OperacionType.Insercion));
        }

        protected override void OnUpdate(TEntity entity)
        {
            if (!this.IsTrackingEnabled)
                return;

            Context.Set<TrackingEntry>().Add(CreateTrackingEntry(entity, TrackingEntry.OperacionType.Actualizacion));
        }

        protected override void OnDelete(TEntity entity)
        {
            if (!this.IsTrackingEnabled)
                return;

            Context.Set<TrackingEntry>().Add(CreateTrackingEntry(entity, TrackingEntry.OperacionType.Borrado));
        }
    }
}
