﻿using LiquidacionGastos.Models;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.Repositories
{
    public class GastoRepository : EntityFrameworkTrackedRepository<Gasto>
    {
        public GastoRepository(LGContext context)
            : base(context) 
        {
        }

        protected override Nullable<Guid> GetGuid(Gasto entity)
        {
            return entity.IdGasto;
        }

        public Gasto CreateGasto(Liquidacion liquidacion)
        {
            Gasto gasto = new Gasto()
            {
                IdGasto = Guid.NewGuid(), 
                Liquidacion = liquidacion                
            };

            return gasto;
        }

        public Gasto GetGasto(Guid idGasto)
        {
            Gasto gasto = this.Get(g => g.IdGasto == idGasto);

            //if (gasto == null)
            //    throw new DataNotFoundException("idGasto", idGasto, "No existe ningún gasto con el valor de idGasto especificado.");

            return gasto;
        }

        //public Gasto GetUsuarioByIdentificador(string identificador)
        //{
        //    Usuario usuario = this.Get(u => u.Identificador == identificador);

        //    if (usuario == null)
        //        throw new DataNotFoundException("identificador", identificador, "No existe ningún usuario con el identificador especificado.");

        //    return usuario;
        //}

        public IQueryable<Gasto> ListGastos()
        {
            return this.ListGastos(null);
        }

        public IQueryable<Gasto> ListGastos(Expression<Func<Gasto, bool>> filterPredicate)
        {
            if (filterPredicate == null)
                filterPredicate = x => true;

            return this.List(filterPredicate)
                       .Include(g=>g.Tipo)
                       .Include(g=>g.CambioMoneda)
                       .Include(g=>g.Liquidacion);
        }

        //public void LoadT(Gasto gasto)
        //{
        //    this.Load(gasto, .T);
        //}

        public void InsertGasto(Gasto gasto)
        {
            this.Insert(gasto);
        }

        public void UpdateGasto(Gasto gasto)
        {
            this.Update(gasto);
        }

        public void DeleteGasto(Gasto gasto)
        {
            this.Delete(gasto);
        }

        public void RefreshGasto(Gasto gasto)
        {
            this.Refresh(gasto);
        }
    }
}
