﻿using LiquidacionGastos.Models;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.Repositories
{
    public class LiquidacionRepository : EntityFrameworkTrackedRepository<Liquidacion>
    {
        //public LiquidacionRepository()
        //    : base()
        //{
        //}

        public LiquidacionRepository(LGContext context)
            : base(context) 
        {
        }

        protected override Nullable<Guid> GetGuid(Liquidacion entity)
        {
            return entity.IdLiquidacion;
        }

        public Liquidacion CreateLiquidacion(Usuario usuario)
        {
            Liquidacion liquidacion = new Liquidacion()
            {
                IdLiquidacion = Guid.NewGuid(),
                Usuario = usuario
            };

            return liquidacion;
        }

        public Liquidacion GetLiquidacion(Guid idLiquidacion)
        {
            Liquidacion liquidacion = this.Get(l => l.IdLiquidacion == idLiquidacion);

            //if (liquidacion == null)
            //    throw new DataNotFoundException("idLiquidacion", idLiquidacion, "No existe ninguna liquidación con el valor de idLiquidacion especificado.");

            return liquidacion;
        }

        //public Liquidacion GetUsuarioByIdentificador(string identificador)
        //{
        //    Usuario usuario = this.Get(u => u.Identificador == identificador);

        //    if (usuario == null)
        //        throw new DataNotFoundException("identificador", identificador, "No existe ningún usuario con el identificador especificado.");

        //    return usuario;
        //}

        public IQueryable<Liquidacion> ListLiquidaciones()
        {
            return this.ListLiquidaciones(null);
        }

        public IQueryable<Liquidacion> ListLiquidaciones(Expression<Func<Liquidacion, bool>> filterPredicate)
        {
            if (filterPredicate == null)
                filterPredicate = x => true;

            return this.List(filterPredicate)
                       .Include(l => l.Usuario)
                       .Include(l => l.Estado)
                       .Include(l => l.Gastos);
        }

        //public void LoadT(Liquidacion liquidacion)
        //{
        //    this.Load(liquidacion, l => l.);
        //}

        public void UpdateImporte(Liquidacion liquidacion)
        {
            if (liquidacion.Gastos == null)
                return;

            decimal total = 0M;
            foreach (Gasto gasto in liquidacion.Gastos)
            {
                total += gasto.Importe / gasto.CambioMoneda.ValorCambio;
            }

            liquidacion.Importe = total;
        }

        public void InsertLiquidacion(Liquidacion liquidacion)
        {
            this.UpdateImporte(liquidacion);
            this.Insert(liquidacion);
        }

        public void UpdateLiquidacion(Liquidacion liquidacion)
        {
            this.UpdateImporte(liquidacion);
            this.Update(liquidacion);
        }

        public void DeleteLiquidacion(Liquidacion liquidacion)
        {
            this.Delete(liquidacion);
        }

        public void RefreshLiquidacion(Liquidacion liquidacion)
        {
            this.Refresh(liquidacion);
        }
    }
}
