﻿using LiquidacionGastos.Models;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.Repositories
{
    public class NotificacionRepository : EntityFrameworkTrackedRepository<Notificacion>
    {
        public NotificacionRepository(LGContext context)
            : base(context)
        {
        }

        protected override Nullable<Guid> GetGuid(Notificacion entity)
        {
            return entity.IdNotificacion;
        }

        public Notificacion CreateNotificacion(CambioEstadoLiquidacion cambioEstado)
        {
            Notificacion notificacion = new Notificacion()
            {
                IdNotificacion = Guid.NewGuid(), 
                CambioEstadoLiquidacion = cambioEstado, 
                EstaNotificado = false, 
                FechaNotificacion = DateTime.Now               
            };

            return notificacion;
        }

        public Notificacion GetNotificacion(Guid idNotificacion)
        {
            Notificacion notificacion = this.Get(d => d.IdNotificacion == idNotificacion);

            //if (notificacion == null)
            //    throw new DataNotFoundException("idNotificacion", idNotificacion, "No existe ningún notificacion con el valor de idNotificacion especificado.");

            return notificacion;
        }

        public IQueryable<Notificacion> ListNotificaciones()
        {
            return this.ListNotificaciones(null);
        }

        public IQueryable<Notificacion> ListNotificaciones(Expression<Func<Notificacion, bool>> filterPredicate)
        {
            if (filterPredicate == null)
                filterPredicate = x => true;

            return this.List(filterPredicate)
                       .Include(t => t.CambioEstadoLiquidacion);
        }

        public void InsertNotificacion(Notificacion notificacion)
        {
            this.Insert(notificacion);
        }

        public void UpdateNotificacion(Notificacion notificacion)
        {
            this.Update(notificacion);
        }

        public void DeleteNotificacion(Notificacion notificacion)
        {
            this.Delete(notificacion);
        }

        public void RefreshNotificacion(Notificacion notificacion)
        {
            this.Refresh(notificacion);
        }
    }

}
