﻿using LiquidacionGastos.Models;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.Repositories
{
    public class UsuarioRepository : EntityFrameworkTrackedRepository<Usuario>
    {
        public UsuarioRepository(LGContext context) : base(context) 
        {
        }

        protected override Nullable<Guid> GetGuid(Usuario entity)
        {
            return entity.IdUsuario;
        }

        /// <summary>
        /// Crear un nuevo objeto Usuario, con IdUsuario único. El nuevo objeto de usuario devuelto no está asociado al repositorio.
        /// Se asocia automáticamente cuando se llame a UpdateUsuario().
        /// </summary>
        /// <returns>El nuevo objeto Usuario que se ha creado.</returns>
        public Usuario CreateUsuario()
        {
            Usuario usuario = new Usuario()
            {
                IdUsuario = Guid.NewGuid()
            };

            return usuario;
        }

        public Usuario GetUsuario(Guid idUsuario)
        {
            Usuario usuario = this.Get(u => u.IdUsuario == idUsuario);

            //if (usuario == null)
            //    throw new DataNotFoundException("idUsuario", idUsuario, "No existe ningún usuario con el valor de idUsuario especificado.");

            return usuario;                
        }

        public Usuario GetUsuarioByIdentificador(string identificador)
        {
            Usuario usuario = this.Get(u => u.Identificador == identificador);

            //if (usuario == null)
            //    throw new DataNotFoundException("identificador", identificador, "No existe ningún usuario con el identificador especificado.");

            return usuario;                
        }

        public IQueryable<Usuario> ListUsuarios()
        {
            return this.ListUsuarios(null);
        }

        public IQueryable<Usuario> ListUsuarios(Expression<Func<Usuario, bool>> filterPredicate)
        {
            if (filterPredicate == null)
                filterPredicate = x => true;

            return this.List(filterPredicate).Include(u => u.Departamento);
        }        

        public void LoadDepartamento(Usuario usuario)
        {
            this.Load(usuario, u => u.Departamento);
        }

        public void LoadSubdepartamentos(Usuario usuario)
        {
            this.LoadCollection(usuario, u => u.DepartamentosSupervisados);
        }

        public void InsertUsuario(Usuario usuario)
        {
            this.Insert(usuario);
        }

        public void UpdateUsuario(Usuario usuario)
        {
            this.Update(usuario);
        }

        public void DeleteUsuario(Usuario usuario)
        {
            this.Delete(usuario);
        }

        public void RefreshUsuario(Usuario usuario)
        {
            this.Refresh(usuario);
        }
    }
}
