﻿using LiquidacionGastos.Framework;
using LiquidacionGastos.Models;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Data.Entity.Validation;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.UnitsOfWork
{
    public class EntityFrameworkUnitOfWork : UnitOfWork, IDisposable
    {
        private LGContext _Context;
        public LGContext Context { get { return _Context; } }

        private readonly bool _IsSharedContext;
        private bool IsSharedContext { get { return _IsSharedContext; } }

        private static object contextCountLock = new object();
        private static int ContextCount = 0;

        public EntityFrameworkUnitOfWork(LGContext context) : base()
        {
            _Context = context;
            _IsSharedContext = true;

            OnInitializeRepositories();
        }

        public EntityFrameworkUnitOfWork()
        {
            LGContext context = ServiceLocator.Resolve<LGContext>();

            if (context != null)
            {
                _Context = context;
                Debug.WriteLine(string.Format("Utilizando contexto único. Número de contextos abiertos: {0}", ContextCount));
            }
            else
            {
                lock (contextCountLock)
                {
                    ContextCount++;
                    Debug.WriteLine(string.Format("Abriendo contexto. Número de contextos abiertos: {0}", ContextCount));
                    _Context = new LGContext();
                }
            }
            
            _IsSharedContext = false;

            OnInitializeRepositories();
        }

        protected override void OnInitialize()
        {            
        }

        protected virtual void OnInitializeRepositories()
        {
        }

        public void Dispose()
        {
            if (this.IsSharedContext)
                return;

            LGContext context = ServiceLocator.Resolve<LGContext>();

            if (context != null)
            {
                _Context = context;
                Debug.WriteLine(string.Format("Dejando de utilizar contexto único. Número de contextos abiertos: {0}", ContextCount));
            }
            else
            {
                _Context.Dispose();
                lock (contextCountLock)
                {
                    ContextCount--;
                    Debug.WriteLine(string.Format("Cerrando contexto. Número de contextos abiertos: {0}", ContextCount));
                }
            }
        }

        public void Commit()
        {
            if (_Context == null)
                throw new NullReferenceException("context");

#if DEBUG
            IEnumerable<DbEntityValidationResult> validationResults = _Context.GetValidationErrors();
            foreach (DbEntityValidationResult result in validationResults)
                foreach(DbValidationError error in result.ValidationErrors)
                    Debug.WriteLine(string.Format("Error de validación de UOW: {0}", error.ErrorMessage));
#endif

            _Context.SaveChanges();
        }

        public void Rollback()
        {
            //_Context.ChangeTracker.Entries().ToList().ForEach(entry => entry.Reload());
        }
    }
}
