﻿using LiquidacionGastos.Models;
using LiquidacionGastos.Repositories;

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.UnitsOfWork
{
    public class MasterTypeUnitOfWork : EntityFrameworkUnitOfWork
    {
        private EstadoLiquidacionRepository EstadoLiquidacionRepository { get; set; }
        private TipoGastoRepository TipoGastoRepository { get; set; }
        private MonedaRepository MonedaRepository { get; set; }
        private CambioMonedaRepository CambioMonedaRepository { get; set; }

        public MasterTypeUnitOfWork() 
        {
        }

        public MasterTypeUnitOfWork(LGContext context) : base(context) 
        {            
        }

        protected override void OnInitializeRepositories()
        {
            this.EstadoLiquidacionRepository = new EstadoLiquidacionRepository(this.Context);
            this.TipoGastoRepository = new TipoGastoRepository(this.Context);
            this.MonedaRepository = new MonedaRepository(this.Context);
            this.CambioMonedaRepository = new CambioMonedaRepository(this.Context);        
        }

        public void CreateMasterTypes()
        {
            this.CreateEstadosLiquidacion();
            this.CreateTiposGasto();
            this.CreateMonedas();
            this.CreateCambiosMoneda();
        }

        private void CreateEstadoLiquidacionHelper(int idEstadoLiquidacion, string nombre)
        {
            EstadoLiquidacion estadoLiquidacion = this.EstadoLiquidacionRepository.CreateEstadoLiquidacion();
            estadoLiquidacion.IdEstadoLiquidacion = idEstadoLiquidacion;
            estadoLiquidacion.Nombre = nombre;
            this.EstadoLiquidacionRepository.InsertEstadoLiquidacion(estadoLiquidacion);
        }

        private void CreateEstadosLiquidacion()
        {
            CreateEstadoLiquidacionHelper((int)EstadoLiquidacionType.Abierta, "Abierta");
            CreateEstadoLiquidacionHelper((int)EstadoLiquidacionType.EsperandoAutorizacion, "Esperando autorización");
            CreateEstadoLiquidacionHelper((int)EstadoLiquidacionType.Rechazada, "Rechazada");
            CreateEstadoLiquidacionHelper((int)EstadoLiquidacionType.Autorizada, "Autorizada");
            CreateEstadoLiquidacionHelper((int)EstadoLiquidacionType.Pagada, "Pagada");
            CreateEstadoLiquidacionHelper((int)EstadoLiquidacionType.Cerrada, "Cerrada");
        }

        private void CreateTipoGastoHelper(int idTipoGasto, string nombre)
        {
            TipoGasto tipoGasto = this.TipoGastoRepository.CreateTipoGasto();
            tipoGasto.IdTipoGasto = idTipoGasto;
            tipoGasto.Nombre = nombre;
            this.TipoGastoRepository.InsertTipoGasto(tipoGasto);
        }

        private void CreateTiposGasto()
        {
            this.CreateTipoGastoHelper((int)TipoGastoType.Billetes, "Billetes de tren, avión, barco o autobús");
            this.CreateTipoGastoHelper((int)TipoGastoType.AlquilerCoche, "Alquiler de coche");            
            this.CreateTipoGastoHelper((int)TipoGastoType.Taxi, "Taxi");
            this.CreateTipoGastoHelper((int)TipoGastoType.Representacion, "Gastos de representación");
            this.CreateTipoGastoHelper((int)TipoGastoType.Formacion, "Formación");
            this.CreateTipoGastoHelper((int)TipoGastoType.Combustible, "Combustible");
            this.CreateTipoGastoHelper((int)TipoGastoType.UsoyAlquilerTlf, "Uso y alquiler de teléfono");
            this.CreateTipoGastoHelper((int)TipoGastoType.Hotel, "Hotel");
            this.CreateTipoGastoHelper((int)TipoGastoType.Kilometraje, "Kilometraje en vehículo propio"); 
            this.CreateTipoGastoHelper((int)TipoGastoType.Dietas, "Dietas diarias");
            this.CreateTipoGastoHelper((int)TipoGastoType.Aparcamiento, "Aparcamiento");
            this.CreateTipoGastoHelper((int)TipoGastoType.Peaje, "Peaje");
            this.CreateTipoGastoHelper((int)TipoGastoType.Portes, "Portes");
            this.CreateTipoGastoHelper((int)TipoGastoType.Otros, "Otros gastos");
        }

        private void CreateMonedaHelper(int idMoneda, string codigoISO4217, string nombre)
        {
            Moneda moneda = this.MonedaRepository.CreateMoneda();
            moneda.IdMoneda = idMoneda;
            moneda.Codigo = codigoISO4217;
            moneda.Nombre = nombre;            
            this.MonedaRepository.InsertMoneda(moneda);
        }

        private void CreateMonedas()
        {
            // Nombres y códigos de las monedas proceden de http://es.wikipedia.org/wiki/ISO_4217
            // El número de clave de la tabla de la base de datos es la posición de la moneda en la tabla referenciada en internet.

            this.CreateMonedaHelper(48, "EUR", "Euro");
            this.CreateMonedaHelper(51, "GBP", "Libra esterlina (libra de Gran Bretaña)");
            this.CreateMonedaHelper(147, "USD", "Dólar estadounidense");
            this.CreateMonedaHelper(71, "JPY", "Yen japonés");
        }

        private void CreateCambioMonedaHelper(int idMoneda, decimal cambio, DateTime fechaCambio)
        {
            CambioMoneda cambioMoneda = this.CambioMonedaRepository.CreateCambioMoneda();
            cambioMoneda.FechaCambio = fechaCambio;
            cambioMoneda.IdMoneda = idMoneda;
            cambioMoneda.ValorCambio = cambio;
            this.CambioMonedaRepository.InsertCambioMoneda(cambioMoneda);
        }

        private void CreateCambiosMoneda()
        {
            this.CreateCambioMonedaHelper(48, 1M, DateTime.Parse("2013/12/07"));
            this.CreateCambioMonedaHelper(51, 0.837922173M, DateTime.Parse("2013/12/07"));
            this.CreateCambioMonedaHelper(147, 1.3695M, DateTime.Parse("2013/12/07"));
            this.CreateCambioMonedaHelper(71, 140.938561M, DateTime.Parse("2013/12/07"));
        }
        
        public EstadoLiquidacion GetEstadoLiquidacion(EstadoLiquidacionType tipo)
        {
            int idEstadoLiquidacion = (int)tipo;
            return this.EstadoLiquidacionRepository.GetEstadoLiquidacion(idEstadoLiquidacion);
        }

        public TipoGasto GetTipoGasto(TipoGastoType tipo)
        {
            int idTipoGasto = (int)tipo;
            return this.TipoGastoRepository.GetTipoGasto(idTipoGasto);
        }

        public ICollection<TipoGasto> ListTiposGasto()
        {
            return this.TipoGastoRepository.ListTiposGasto().ToList();
        }

        public CambioMoneda GetCambioMoneda(string codigoMoneda, DateTime fechaCambio)
        {
            return this.CambioMonedaRepository.GetCambioModenaByCodigo(codigoMoneda, fechaCambio);
        }

        public CambioMoneda GetCambioMoneda(string codigoMoneda)
        {
            return this.GetCambioMoneda(codigoMoneda, DateTime.Today);
        }

        public ICollection<CambioMoneda> ListCambiosMonedaMostRecent()
        {
            return this.CambioMonedaRepository.ListCambiosMonedaMostRecent().ToList();
        }
    }

}
