﻿using LiquidacionGastos.Models;
using LiquidacionGastos.Repositories;
using LiquidacionGastos.UnitsOfWork;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.Security
{
    public class UserSecurity
    {
        public UserSecurity()
        {
        }

        public Usuario AuthenticateUser(string identificador, string passwd)
        {
            Usuario u = null;

            using (OrganizacionUnitOfWork uow = new OrganizacionUnitOfWork())
            {
                u = uow.GetUsuarioByIdentificador(identificador);

                // Si el usuario existe cargar su estructura de departamentos asociada.
                if (u != null)
                {
                    uow.LoadDepartamento(u);
                    uow.LoadSubdepartamentos(u);
                }
            }
            
            // Si el usuario no existe, terminar con valor null.
            if (u == null || u.HashContrasena == null)
                return null;

            // Si el usuario no tiene activado el login, terminar con valor null.
            if (!u.EstaActivadoLogin)
                return null;

            // Si la contraseña no coincide, terminar con valor null.
            if (!PasswordHash.ValidatePassword(passwd, u.HashContrasena))
                return null;

            // Todos los controles son correctos, devolver el objeto usuario solicitado.
            return u;
        }

    }
}
