﻿using LiquidacionGastos.Models;
using LiquidacionGastos.Repositories;
using LiquidacionGastos.UnitsOfWork;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.UnitsOfWork
{
    public class TrackingUnitOfWork : EntityFrameworkUnitOfWork
    {
        private TrackingEntryRepository TrackingEntryRepository { get; set; }

        public TrackingUnitOfWork()
        {
        }

        public TrackingUnitOfWork(LGContext context) : base(context) 
        {            
        }

        protected override void OnInitializeRepositories()
        {
            this.TrackingEntryRepository = new TrackingEntryRepository(this.Context);
        }

        public TrackingEntry GetTrackingEntry(Guid idTrackingEntry)
        {
            if (idTrackingEntry == null)
                throw new ArgumentNullException("idTrackingEntry");

            return this.TrackingEntryRepository.GetTrackingEntry(idTrackingEntry);
        }

        public IList<TrackingEntry> ListTrackingEntries(DateTime dateFrom)
        {
            return this.TrackingEntryRepository.ListTrackingEntries(t=> t.Fecha >= dateFrom )
                                               .OrderBy(t => t.Fecha)
                                               .ToList();
        }

        public void InsertTrackingEntry(TrackingEntry trackingEntry)
        {
            if (trackingEntry == null)
                throw new ArgumentNullException("trackingEntry");

            this.TrackingEntryRepository.InsertTrackingEntry(trackingEntry);
        }

        public void UpdateTrackingEntry(TrackingEntry trackingEntry)
        {
            if (trackingEntry == null)
                throw new ArgumentNullException("trackingEntry");

            this.TrackingEntryRepository.UpdateTrackingEntry(trackingEntry);
        }

        public void DeleteTrackingEntry(TrackingEntry trackingEntry)
        {
            if (trackingEntry == null)
                throw new ArgumentNullException("trackingEntry");

            this.TrackingEntryRepository.DeleteTrackingEntry(trackingEntry);
        }
    }
}
