﻿using LiquidacionGastos.Business;
using LiquidacionGastos.Framework;
using LiquidacionGastos.Models;
using LiquidacionGastos.Repositories;
using LiquidacionGastos.Services;
using LiquidacionGastos.Synch;
using LiquidacionGastos.ViewModels;
using MahApps.Metro;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Markup;

namespace LiquidacionGastos
{
    /// <summary>
    /// Lógica de interacción para App.xaml
    /// </summary>
    public partial class App : Application
    {
        protected override void OnStartup(StartupEventArgs e)
        {
            // Iniciar ventana de splash.
            SplashScreen splashScreen = new SplashScreen("pictures/splashscreen.png");
            splashScreen.Show(true, false);

            // Código utilizado para testear la cultura con otras culturas.
            //System.Threading.Thread.CurrentThread.CurrentCulture =
            //    System.Threading.Thread.CurrentThread.CurrentUICulture =
            //        new System.Globalization.CultureInfo("es-ES");

            // Por defecto, WPF utiliza la cultura en-US con idependencia de la configuración del sistema operativo.
            // La siguiente llamada asegura que la cultura utilizada en los bindings es la del sistema operativo.
            FrameworkElement.LanguageProperty.OverrideMetadata(
              typeof(FrameworkElement),
              new FrameworkPropertyMetadata(XmlLanguage.GetLanguage(CultureInfo.CurrentCulture.IetfLanguageTag))
            );

            // Ejecutar las funciones de inicio predeterminadas de aplicación.
            base.OnStartup(e);

            // Iniciar los servicios de la aplicación.
            ServiceLocator.Register<IAuthenticationService>(new Services.AuthenticationService());
            ServiceLocator.Register<IAuthorizationService>(new AuthorizationService());
            ServiceLocator.Register<Synchronizer>(new Synchronizer());
            ServiceLocator.Register<LGContext>(new LGContext());

            // Crear la ventana principal de aplicación.
            MainWindow mainWindow = new MainWindow();

            // Crear el ViewModel principal de la aplicación.
            MainWindowViewModel viewModel = new MainWindowViewModel();

            // Asociar
            mainWindow.Closing += (sender, ev) => viewModel.Stop();
            mainWindow.DataContext = viewModel;
            mainWindow.WindowStartupLocation = WindowStartupLocation.CenterScreen;

            ApplicationServices.NavigationService.ContentPlace = mainWindow.TransitionBox;
            Application.Current.MainWindow = mainWindow;            

            mainWindow.Show();
            viewModel.Login();            
        }

    }
}
