﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.Framework
{
    public class VectorCalendarHelper
    {
        public static byte[] ResetVectorCalendar(byte[] vectorCalendar)
        {
            vectorCalendar = new byte[vectorCalendar.Length];
            return vectorCalendar;
        }

        public static int DecodeYear(byte[] vectorCalendar)
        {
            return (vectorCalendar[0] << 12) + (vectorCalendar[1] << 8) + (vectorCalendar[2] << 4) + (vectorCalendar[3]);
        }

        public static byte[] EncodeYear(byte[] vectorCalendar, int year)
        {
            vectorCalendar[0] = (byte)((year & (0xf << 12)) >> 12);
            vectorCalendar[1] = (byte)((year & (0xf << 8)) >> 8);
            vectorCalendar[2] = (byte)((year & (0xf << 4)) >> 4);
            vectorCalendar[3] = (byte)(year & (0xf));

            return vectorCalendar;
        }

        public static ObservableCollection<DateTime> DecodeDates(ObservableCollection<DateTime> Calendar, byte[] vectorCalendar)
        {
            int year = DecodeYear(vectorCalendar);

            IList<DateTime> dates = new List<DateTime>();
            int dayOfYear = 0;
            for (int p = 4; p < vectorCalendar.Length; p++)
            {
                byte b = vectorCalendar[p];

                for (int j = 7; j >= 0; j--)
                {
                    dayOfYear++;

                    int bitMask = 1 << j;
                    bool esDiaFestivo = ((b & bitMask) == bitMask);

                    if (!esDiaFestivo)
                        continue;

                    DateTime date = new DateTime(year, 1, 1).AddDays(dayOfYear - 1);
                    dates.Add(date);
                }
            }
            Calendar.ReplaceRange(dates);
            return Calendar;
        }

        public static byte[] EncodeDates(byte[] vectorCalendar, ObservableCollection<DateTime> Calendar)
        {
            int year = DecodeYear(vectorCalendar);

            vectorCalendar = ResetVectorCalendar(vectorCalendar);
            vectorCalendar = EncodeYear(vectorCalendar, year);

            foreach (DateTime date in Calendar)
            {
                if (year == 0)
                {
                    EncodeYear(vectorCalendar, date.Year);
                }
                else
                    if (date.Year != year)
                        throw new ApplicationException("No se puede codificar fechas de diferentes años.");

                int n = date.DayOfYear;

                int p = (n / 8) + 4;
                int j = 8 - (n % 8);

                byte b = vectorCalendar[p];
                byte bitsMask = (byte)(1 << j);
                vectorCalendar[p] = (byte)(vectorCalendar[p] | bitsMask);
            }

            return vectorCalendar;
        }

        public static byte[] ExchangeDaysVectorCalendar(byte[] vectorCalendar)
        {
            int year = DecodeYear(vectorCalendar);

            IList<DateTime> dates = new List<DateTime>();
            int dayOfYear = 0;
            for (int p = 4; p < vectorCalendar.Length; p++)
            {
                byte b = vectorCalendar[p];

                for (int j = 7; j >= 0; j--)
                {
                    dayOfYear++;

                    int bitMask = 1 << j;
                    bool esDiaFestivo = ((b & bitMask) == bitMask);

                    if (!esDiaFestivo)
                    {
                        byte bitsMask = (byte)(1 << j);
                        vectorCalendar[p] = (byte)(vectorCalendar[p] | bitsMask);
                    }
                    else
                    {
                        byte bitsMask = (byte)(0 << j);
                        vectorCalendar[p] = (byte)(vectorCalendar[p] & bitsMask);
                    }
                }
            }
            return vectorCalendar;
        }

    }
}
