﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace LiquidacionGastos.Framework
{    
    public class WindowNotification : Window
    {
        public static void Show(string text)
        {
            WindowNotification.Show(text, TimeSpan.FromSeconds(1));
        }

        public static void Show(string text, TimeSpan timeSpan)
        {
            WindowNotification wn = new WindowNotification(text);
            wn.Duration = timeSpan;
            wn.Show();
        }

        public static void Show(Window owner, string text, TimeSpan timeSpan)
        {
            WindowNotification wn = new WindowNotification(owner, text);
            wn.Duration = timeSpan;
            wn.Show();
        }

        public string Text { get; private set; }
        public Label Label { get; private set; }
        public TimeSpan Duration { get; set; }

        private WindowNotification(string text) : this(Application.Current.MainWindow, text)
        {            
        }

        private WindowNotification(Window owner, string text)
        {
            this.Text = text;

            this.Owner = owner;
            this.ResizeMode = ResizeMode.NoResize;
            this.WindowStyle = WindowStyle.None;            
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;
            this.ShowInTaskbar = false;
            this.AllowsTransparency = true;
            this.Background = new SolidColorBrush(Colors.Transparent);
            this.Opacity = 0.0;

            this.Label = new Label();            
            this.Label.Content = text;
            FormattedText formattedText = new FormattedText(text,
                                              Thread.CurrentThread.CurrentCulture,
                                              FlowDirection.LeftToRight,
                                              new Typeface(this.Label.FontFamily.ToString()),
                                              this.Label.FontSize,
                                              Brushes.Black);

            this.Width = Math.Max(formattedText.Width, 100) + 40;
            this.Height = Math.Max(formattedText.Height, 30) + 30;

            FontFamily ff = new FontFamily("Segoe UI Semibold");
            if (ff != null)
                this.Label.FontFamily = ff;

            this.Label.Margin = new Thickness(0);
            this.Label.HorizontalAlignment = HorizontalAlignment.Center;
            this.Label.VerticalAlignment = VerticalAlignment.Center;
            this.Label.HorizontalContentAlignment = System.Windows.HorizontalAlignment.Center;
            this.Label.VerticalContentAlignment = System.Windows.VerticalAlignment.Center;
            this.Label.Width = this.Width;
            this.Label.Height = this.Height;
            this.Label.Foreground = new SolidColorBrush(Colors.White);
            this.Label.Background = Brushes.Transparent;

            Rectangle myRectangle = new Rectangle();
            myRectangle.Fill = new SolidColorBrush(Color.FromArgb(0xff, 0x8, 0x6f, 0x9e));
            myRectangle.Opacity = 0.9;
            myRectangle.StrokeThickness = 2;
            myRectangle.Stroke = Brushes.White;
            myRectangle.RadiusX = 0.0; //20.0;
            myRectangle.RadiusY = 0.0; //20.0;
            myRectangle.Width = this.Width;
            myRectangle.Height = this.Height;

            Grid g = new Grid();
            this.AddChild(g);

            g.Children.Add(myRectangle);
            g.Children.Add(this.Label);
        }

        private new void Show()
        {
            base.Show();
            this.Fade(1.0, 0.5, this.da_Completed);
        }

        private DoubleAnimation da;
        private void Fade(double fadeTo, double seconds, EventHandler ev)
        {
            da = new DoubleAnimation();
            da.To = fadeTo;
            da.By = 1.0;
            da.Duration = new Duration(TimeSpan.FromSeconds(seconds));
            da.Completed += ev;
            this.BeginAnimation(Window.OpacityProperty, da);
            
        }

        DispatcherTimer t;

        void da_Completed(object sender, EventArgs e)
        {
            da.Completed -= this.da_Completed;
            t = new DispatcherTimer();
            t.Tick += t_Tick;
            t.Interval = this.Duration;
            t.Start();
        }

        void da_FadeOut(object sender, EventArgs e)
        {
            da.Completed -= this.da_FadeOut;
            Close();
        }


        void t_Tick(object sender, EventArgs e)
        {
            t.Stop();
            this.Fade(0.0, 0.5, this.da_FadeOut);
        }
    }
}
