﻿using LiquidacionGastos.Framework;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.Services
{
    public enum ApplicationStates : int
    {
        Login,
        LiquidacionList,
        Shutdown
    };

    public class StateChangedEventArgs : EventArgs
    {
        public ApplicationStates PreviousState { get; set; }
        public ApplicationStates CurrentState { get; set; }
    }

    public class ApplicationStateService
    {
        public event EventHandler<StateChangedEventArgs> StateChanged;

        protected void OnStateChanged(ApplicationStates previousState, ApplicationStates currentState)
        {
            EventHandler<StateChangedEventArgs> handler = this.StateChanged;
            if (handler != null)
            {
                var e = new StateChangedEventArgs() { PreviousState = previousState, CurrentState = currentState };
                handler(this, e);
            }
        }

        private StateCommand<ApplicationStates> StateContext { get; set; }

        public ApplicationStates CurrentState
        {
            get { return this.StateContext.CurrentStateId; }
            set { this.StateContext.CurrentStateId = value; }
        }

        public ApplicationStates PreviousState
        {
            get { return this.StateContext.PreviousStateId; }
        }        

        public ApplicationStateService()
        {
            this.StateContext = new StateCommand<ApplicationStates>();
            this.StateContext.AddState(ApplicationStates.Login, OnSetState, ApplicationStates.LiquidacionList, ApplicationStates.Shutdown);
            this.StateContext.AddState(ApplicationStates.LiquidacionList, OnSetState, ApplicationStates.LiquidacionList, ApplicationStates.Shutdown);
            this.StateContext.AddState(ApplicationStates.Shutdown, OnSetState);
        }

        private void OnSetState(StateCommand<ApplicationStates> stateContext)
        {
            OnStateChanged(stateContext.PreviousStateId, stateContext.CurrentStateId);
        }
    }
}
