﻿#define REPOSITORIES_SIMULATED

using LiquidacionGastos.Models;
using LiquidacionGastos.Security;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Security;
using System.Security.Claims;
using System.Security.Cryptography;
using System.Security.Permissions;
using System.Security.Principal;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace LiquidacionGastos.Services
{
    public class AuthenticationService : IAuthenticationService
    {
        private GenericPrincipal _Principal;
        public GenericPrincipal Principal
        {
            get { return _Principal; }
            private set { _Principal = value; }
        }
        
        public IUsuarioPrincipal UsuarioPrincipal
        {
            get { return _Principal as IUsuarioPrincipal; }
        }

        public AuthenticationService()
        {
            GenericPrincipal.ClaimsPrincipalSelector = GetPrincipal;
        }

        public ClaimsPrincipal GetPrincipal()
        {
            return _Principal;
        }

        /// <summary>
        /// Invalida las credenciales del usuario actual.
        /// </summary>
        public void Logout()
        {
            // Crea una identidad vacia.
            IIdentity identity = new GenericIdentity(string.Empty);

            // Crear el objeto principal con la identidad vacia y asignarlo al contexto de la aplicación.
            this.Principal = new GenericPrincipal(identity, null);
        }

        public bool AuthenticateUser(string identificador, string passwd)
        {
            // Asegurar que se liberan las credenciales anteriores.
            this.Logout();

            // Crear el objeto de gestión de seguridad de usuario.
            UserSecurity userSecurity = new UserSecurity();

            // Validar las credenciales de usuario.
            Usuario usuario = userSecurity.AuthenticateUser(identificador, passwd);

            // Si las credenciales no han sido correctamente validadas, no se dispone del objeto de usuario.
            if (usuario == null)
                return false;

            // Crea el objeto Identity para el usuario actual.
            IIdentity identity = new GenericIdentity(usuario.Identificador, "basic");
            
            // Crea una lista vacia de roles del usuario.
            List<string> roles = new List<string>();

            // Añade el role de usuario si el usuario pertenece a algún departamento.
            if (usuario.Departamento != null)
                roles.Add(UsuarioRoles.Usuario.ToString());

            // Si el usuario supervisa departamentos, añade el rol de Autorizador.
            if (usuario.DepartamentosSupervisados.Count > 0)
                roles.Add(UsuarioRoles.Autorizador.ToString());
            
            // Si el usuario tiene perfil de administrador, añade el role a la lista.
            if (usuario.EsAdministrador)
                roles.Add(UsuarioRoles.Administrador.ToString());

            // Crear el objeto principal y lo asigna al contexto de la aplicación.
            this.Principal = new UsuarioPrincipal(usuario, identity, roles.ToArray());

            // Indicar que la autenticación ha sido correcta.
            return true;
        }

        private void DemandGenericPrincipalPermission(string name, string role, bool isAuthenticated)
        {
            IPrincipal previousPrincipal = Thread.CurrentPrincipal;
            try
            {
                Thread.CurrentPrincipal = GenericPrincipal.Current;
                PrincipalPermission permission = new PrincipalPermission(name, role, isAuthenticated);
                permission.Demand();
            }
            catch (SecurityException ex)
            {
                throw ex;
            }
            finally
            {
                Thread.CurrentPrincipal = previousPrincipal;
            }
        }

    }
}
