﻿using LiquidacionGastos.ViewModels;
using LiquidacionGastos;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using LiquidacionGastos.Views;
using MahApps.Metro.Controls;

namespace LiquidacionGastos.Services
{
    public enum ViewName : int
    {
        LoginView = 1,
        LiquidacionListView,
        LiquidacionView,
        GastoView,
        UsuarioListView,
        UsuarioView,
        DepartamentoListView,        
        DepartamentoView,
        CalendariosView,
        OpcionesView,
        ConfirmacionView,
        DeleteUsuarioMessageView,
        DeleteDepartamentoMessageView,
        DeleteGastoMessageView
    }

    public enum ViewBehavior : int
    {
        Normal = 1,
        Popup,
        MessageDialog
    }

    public class NavigationService
    {
        public TransitioningContentControl ContentPlace { get; set; }

        private readonly IDictionary<IViewModel, IDictionary<ViewName, UserControl>> ViewsRegistry;
        
        private readonly Stack<UserControl> ChildViewList;

        private readonly Dictionary<ViewName, NavigationViewInfo> ViewInfoList;

        public NavigationService()
        {
            this.ViewInfoList = new Dictionary<ViewName, NavigationViewInfo>();

            this.ViewInfoList.Add(ViewName.LoginView, new NavigationViewInfo() { ViewId = ViewName.LoginView, ViewType = typeof(LoginView), ViewBehavior = ViewBehavior.Normal });
            this.ViewInfoList.Add(ViewName.LiquidacionListView, new NavigationViewInfo() { ViewId = ViewName.LiquidacionListView, ViewType = typeof(LiquidacionListView), ViewBehavior = ViewBehavior.Normal });
            this.ViewInfoList.Add(ViewName.LiquidacionView, new NavigationViewInfo() { ViewId = ViewName.LiquidacionView, ViewType = typeof(LiquidacionView), ViewBehavior = ViewBehavior.Popup });
            this.ViewInfoList.Add(ViewName.GastoView, new NavigationViewInfo() { ViewId = ViewName.GastoView, ViewType = typeof(GastoView), ViewBehavior = ViewBehavior.Popup });
            this.ViewInfoList.Add(ViewName.UsuarioListView, new NavigationViewInfo() { ViewId = ViewName.UsuarioListView, ViewType = typeof(UsuarioListView), ViewBehavior = ViewBehavior.Popup });
            this.ViewInfoList.Add(ViewName.UsuarioView, new NavigationViewInfo() { ViewId = ViewName.UsuarioView, ViewType = typeof(UsuarioView), ViewBehavior = ViewBehavior.Popup });
            this.ViewInfoList.Add(ViewName.DepartamentoListView, new NavigationViewInfo() { ViewId = ViewName.DepartamentoListView, ViewType = typeof(DepartamentoListView), ViewBehavior = ViewBehavior.Popup });
            this.ViewInfoList.Add(ViewName.DepartamentoView, new NavigationViewInfo() { ViewId = ViewName.DepartamentoView, ViewType = typeof(DepartamentoView), ViewBehavior = ViewBehavior.Popup });
            this.ViewInfoList.Add(ViewName.CalendariosView, new NavigationViewInfo() { ViewId = ViewName.CalendariosView, ViewType = typeof(CalendariosView), ViewBehavior = ViewBehavior.Popup });
            this.ViewInfoList.Add(ViewName.OpcionesView, new NavigationViewInfo() { ViewId = ViewName.OpcionesView, ViewType = typeof(OpcionesView), ViewBehavior = ViewBehavior.Normal });
            this.ViewInfoList.Add(ViewName.ConfirmacionView, new NavigationViewInfo() { ViewId = ViewName.ConfirmacionView, ViewType = typeof(ConfirmacionView), ViewBehavior = ViewBehavior.MessageDialog });
            this.ViewInfoList.Add(ViewName.DeleteUsuarioMessageView, new NavigationViewInfo() { ViewId = ViewName.DeleteUsuarioMessageView, ViewType = typeof(DeleteUsuarioMessageView), ViewBehavior = ViewBehavior.MessageDialog });
            this.ViewInfoList.Add(ViewName.DeleteDepartamentoMessageView, new NavigationViewInfo() { ViewId = ViewName.DeleteDepartamentoMessageView, ViewType = typeof(DeleteDepartamentoMessageView), ViewBehavior = ViewBehavior.MessageDialog });
            this.ViewInfoList.Add(ViewName.DeleteGastoMessageView, new NavigationViewInfo() { ViewId = ViewName.DeleteGastoMessageView, ViewType = typeof(DeleteGastoMessageView), ViewBehavior = ViewBehavior.MessageDialog });
           
            this.ViewsRegistry = new Dictionary<IViewModel, IDictionary<ViewName, UserControl>>();
            this.ChildViewList = new Stack<UserControl>();
        }

        public bool RegisterView(IViewModel viewModel, ViewName viewId)
        {
            // Comprobar si existe registro para el viewModel indicado, y crearlo si no existe.
            if (!this.ViewsRegistry.ContainsKey(viewModel))
                this.ViewsRegistry.Add(viewModel, new Dictionary<ViewName, UserControl>());

            // Obtener diccionario de vistas creadas del viewModel.
            IDictionary<ViewName, UserControl> viewsOfviewModel = this.ViewsRegistry[viewModel];

            // Si no está registrada la vista en el diccionario del viewModel, crear la vista y registrarla.
            if (!viewsOfviewModel.ContainsKey(viewId))
            {
                Type viewType = this.ViewInfoList[viewId].ViewType;
                UserControl view = CreateView(viewType) as UserControl;
                view.DataContext = viewModel;
                viewsOfviewModel.Add(viewId, view);
                return true;
            }

            return false;
        }

        public bool UnregisterView(IViewModel viewModel, ViewName viewId)
        {
            if (GetView(viewModel, viewId) == ContentPlace.Content)
            {
                this.ContentPlace.Transition = "RightReplaceTransition";

                if (this.ViewInfoList[viewId].ViewBehavior == ViewBehavior.Normal)
                {
                    ContentPlace.Content = null;
                }
                else if (this.ViewInfoList[viewId].ViewBehavior == ViewBehavior.Popup)
                {
                    if (this.ChildViewList.Count() > 0 )
                        ContentPlace.Content = this.ChildViewList.Pop();
                    else
                        ContentPlace.Content = null;
                }
            }

            // Comprobar si existe registro para el viewModel indicado.
            if (!this.ViewsRegistry.ContainsKey(viewModel))
                return false;

            // Obtener diccionario de vistas creadas del viewModel.
            IDictionary<ViewName, UserControl> viewsOfviewModel = this.ViewsRegistry[viewModel];

            if (!viewsOfviewModel.ContainsKey(viewId))
                return false;

            // Eliminar el usercontrol del registro de vistas del viewModel.
            viewsOfviewModel.Remove(viewId);

            // Si el viewModel se quedó sin vistas registradas, eliminar del regitro del viewModel también.
            if (viewsOfviewModel.Count == 0)
                this.ViewsRegistry.Remove(viewModel);

            return true;
        }

        public UserControl GetView(IViewModel viewModel, ViewName viewId)
        {
            // Comprobar si existe registro para el viewModel indicado.
            if (!this.ViewsRegistry.ContainsKey(viewModel))
                return null;

            // Obtener diccionario de vistas creadas del viewModel.
            IDictionary<ViewName, UserControl> viewsOfviewModel = this.ViewsRegistry[viewModel];

            // Comprobar Si está registrada la vista en el diccionario del viewModel.
            if (!viewsOfviewModel.ContainsKey(viewId))
                return null;

            // Devolver el usercontrol.
            return viewsOfviewModel[viewId];            
        }

        private object CreateView(Type type)
        {
            object o = null;
            
            try{
                o = Activator.CreateInstance(type, null);
            }catch(Exception ex) {
                throw new TypeLoadException(string.Format("No se puede cargar el tipo {0}", type.Name), ex);
            }

            return o;
        }
        
        public void Navigate(IViewModel viewModel, ViewName viewId)
        {
            if (this.ContentPlace == null)
                throw new NullReferenceException("No se inicializó la propiedad ContentPlace");

            if (viewModel == null)
                throw new ArgumentNullException("viewModel", "El parametro es null");

            if (!this.ViewInfoList.ContainsKey(viewId))
                throw new ArgumentOutOfRangeException("view", "No se puede recuperar el tipo de vista indicada.");

            UserControl view = GetView(viewModel, viewId);

            if (view == null)
                throw new NullReferenceException(string.Format("La vista {0} no está registrada. Llame antes al método RegisterView().", viewId.ToString()));

            NavigationViewInfo viewInfo = this.ViewInfoList[viewId];

            if (viewInfo.ViewBehavior == ViewBehavior.Popup)
            {
                this.ChildViewList.Push((UserControl)this.ContentPlace.Content);
            }
            else
            {
                this.ChildViewList.Clear();
            }
            
            // Establecer el efecto visual de transición.
            ContentPlace.Transition = "LeftReplaceTransition";

            // Ajustar las dimensiones de la vista al contenedor.
            view.Width = double.NaN;
            view.Height = double.NaN;

            // Ejecutar la transición.
            ContentPlace.Content = view;           
        }
    }
}
