﻿using LiquidacionGastos.Framework;
using LiquidacionGastos.Models;
using LiquidacionGastos.Services;
using LiquidacionGastos.WebServiceRepositories;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace LiquidacionGastos.Synch
{
    public class Synchronizer
    {

        #region [ServerNotReached Event]

        public event EventHandler<System.EventArgs> ServerNotReached;

        private void OnServerNotReached()
        {
            EventHandler<System.EventArgs> handler = this.ServerNotReached;

            if (handler != null)
                handler(this, new EventArgs());            
        }

        private void CallFromMainThreadOnServerNotReached()
        {
            App.Current.Dispatcher.Invoke((Action)OnServerNotReached);            
        }

        #endregion [ServerNotReached Event]

        #region [SynchronizationCompleted Event]

        public event EventHandler<System.EventArgs> SynchronizationCompleted;

        private void OnSynchronizationCompleted()
        {
            EventHandler<System.EventArgs> handler = this.SynchronizationCompleted;

            if (handler != null)
                handler(this, new EventArgs());            
        }

        private void CallFromMainThreadOnSynchronizationCompleted()
        {
            App.Current.Dispatcher.Invoke((Action)OnSynchronizationCompleted);            
        } 

        #endregion [ServerNotReached Event]

        private object thislock;
        private Thread WorkerThread;        
        private Worker WorkerObject;        
        
        public Synchronizer()
        {
            thislock = new object();
            this.WorkerObject = new Worker(this);
            this.WorkerThread = null;
        }

        public bool Start()
        {
            lock (thislock)
            {
                if (this.WorkerThread != null)
                {
                    var t = this.WorkerThread.ThreadState;
                    if (this.WorkerThread.ThreadState == System.Threading.ThreadState.Stopped)
                        this.WorkerThread = null;
                    else
                        return false;
                }

                this.WorkerThread = new Thread(this.WorkerObject.DoWork);
                this.WorkerThread.Start();
                return true;
            }
        }

        public void Stop()
        {
            lock (thislock)
            {
                if (this.WorkerThread == null)
                    return;

                this.WorkerObject.RequestStop();
                var t = this.WorkerThread.ThreadState;
                this.WorkerThread.Join(4000);
                this.WorkerThread = null;
            }
        }

        public class Worker
        {
            private bool _Error;
            private bool _shouldStop;
            private Synchronizer _Synchronizer;

            public Worker(Synchronizer synchonizer)
            {
                _Error = false;
                _shouldStop = false;
                _Synchronizer = synchonizer;
            }

            // This method will be called when the thread is started.
            public void DoWork()
            {
                Debug.WriteLine(string.Format("Iniciando subproceso {0}", Thread.CurrentThread.ManagedThreadId));
                _Error = false;
                _shouldStop = false;
                while (!_shouldStop)
                {
                    try
                    {                        
                        CompleteSyncUnitOfWork uow = new CompleteSyncUnitOfWork();
                        DateTime dt1 = DateTime.Now;
                        long entitySynchronizedCount = uow.Syncronize();
                        DateTime dt2 = DateTime.Now;
                        Debug.WriteLine(string.Format("Inicio: {0}", dt1.ToLongTimeString()));
                        Debug.WriteLine(string.Format("Fin: {0}", dt2.ToLongTimeString()));
                        Debug.WriteLine(string.Format("Lapso: {0}", (dt2-dt1).TotalSeconds.ToString()));

                        if (entitySynchronizedCount > 0)
                            this._Synchronizer.CallFromMainThreadOnSynchronizationCompleted();

                        Thread.Sleep(30000);
                    }
                    catch (EndpointNotFoundException ex)
                    {
                        // El servidor está desconectado.
                        Debug.WriteLine("Excepción capturada: No se puede conectar con el servidor");
                        this.RequestStop();
                        _Error = true;                        
                    }
                }
                if (_Error)
                    this._Synchronizer.CallFromMainThreadOnServerNotReached();                    

                Debug.WriteLine(string.Format("Subproceso {0} terminado", Thread.CurrentThread.ManagedThreadId));
            }

            public void RequestStop()
            {
                _shouldStop = true;
            }

        }
    }
}
