﻿using LiquidacionGastos.Business;
using LiquidacionGastos.Framework;
using LiquidacionGastos.Models;
using LiquidacionGastos.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.ViewModels
{
    public class CalendariosViewModel : ViewModel
    {
        #region [Subsistema básico]

        public RelayCommand CancelCommand { get; set; }

        public CalendariosViewModel()
        {
            this.ViewId = ViewName.CalendariosView;
            this.SeleccionDiasLaborables = new ObservableCollection<DateTime>();
            this.DiasLaborables = new ObservableCollection<DateTime>();
            this.SeleccionDiasFestivos = new ObservableCollection<DateTime>();
            this.DiasFestivos = new ObservableCollection<DateTime>();
            this.MesActual = DateTime.Parse(DateTime.Today.ToString("yyyy/MM/01"));
            
            this.CancelCommand = new RelayCommand(OnCancel);
            this.SetSelectionAsDiaLaborableCommand = new RelayCommand(OnSetSelectionAsDiaLaborable, OnCanSetSelectionAsDiaLaborable);
            this.SetSelectionAsDiaFestivoCommand = new RelayCommand(OnSetSelectionAsDiaFestivo, OnCanSetSelectionAsDiaFestivo);
            this.CreateCalendarioCommand = new RelayCommand(OnCreateCalendario);
            this.DeleteCalendarioCommand = new RelayCommand(OnDeleteCalendario, OnCanDeleteCalendario);
            this.DuplicateCalendarioCommand = new RelayCommand(OnDuplicateCalendario, OnCanDuplicateCalendario);
            this.ResetCalendarioCommand = new RelayCommand(OnResetCalendario, OnCanResetCalendario);
            this.SaveCommand = new RelayCommand(OnSave, OnCanSave);

            this.CalendarioList.AddRange(this.CalendarioManagent.ListCalendarios());

            if (this.CalendarioList.Count() == 0)
                this.OnCreateCalendario(null);
            else
                this.CurrentCalendario = this.CalendarioList.FirstOrDefault();
        }

        private void OnCancel(object param)
        {
            this.Stop(0);
        }

        #endregion [Subsistema básico]

        #region [Subsistema de mantenimiento]
        
        private Calendario _CurrentCalendario;
        public Calendario CurrentCalendario 
        { 
            get { return _CurrentCalendario; } 
            set { 
                _CurrentCalendario = value; 

                if(CurrentCalendarioForEdition != value) 
                    this.CurrentCalendarioForEdition = value;

                if (_CurrentCalendario != null)
                {
                    byte[] v = this._CurrentCalendario.DiasFestivos.ToArray();
                    int year = VectorCalendarHelper.DecodeYear(v);
                    if (year == 256)
                        v = VectorCalendarHelper.EncodeYear(v, this.MesActual.Year);
;                    this.DiasFestivos = VectorCalendarHelper.DecodeDates(this.DiasFestivos, v);
                    this.DiasLaborables = VectorCalendarHelper.DecodeDates(this.DiasLaborables, VectorCalendarHelper.ExchangeDaysVectorCalendar(v));
                }

                OnPropertyChanged(); 
            } 
        }

        // Propiedad que actua como buffer en la edición del control ComboBox.
        // Mientras se edita el combobox CurentCalendarForEdition es establecida a null
        // por el control Combobox, pero esta propiedad NO modifica el calendario seleccionado actualmente
        // a menos que se seleccione otro de la lista, en cuyo caso, su valor NO es null.
        private Calendario _CurrentCalendarioForEdition;
        public Calendario CurrentCalendarioForEdition 
        { 
            get { return _CurrentCalendarioForEdition; } 
            set { _CurrentCalendarioForEdition = value; 
                if (value != null) 
                    this.CurrentCalendario = value; 
            } 
        }
                
        public string Nombre 
        { 
            get { return this.CurrentCalendario.Nombre; }
            set { this.CurrentCalendario.Nombre = value; } 
        }
        
        private DateTime _MesActual;
        public DateTime MesActual { get { return _MesActual; } set { _MesActual = value; OnPropertyChanged(); } }
                
        private ObservableCollection<DateTime> _SeleccionDiasLaborables;
        public ObservableCollection<DateTime> SeleccionDiasLaborables { get { return _SeleccionDiasLaborables; } set { _SeleccionDiasLaborables = value; OnPropertyChanged(); } }
                
        private ObservableCollection<DateTime> _DiasLaborables;
        public ObservableCollection<DateTime> DiasLaborables { get { return _DiasLaborables; } set { _DiasLaborables = value; OnPropertyChanged(); } }
                
        private ObservableCollection<DateTime> _SeleccionDiasFestivos;
        public ObservableCollection<DateTime> SeleccionDiasFestivos { get { return _SeleccionDiasFestivos; } set { _SeleccionDiasFestivos = value; OnPropertyChanged(); } }
        
        private ObservableCollection<DateTime> _DiasFestivos;
        public ObservableCollection<DateTime> DiasFestivos { get { return _DiasFestivos; } set { _DiasFestivos = value; OnPropertyChanged(); } }
                                
        private ObservableCollection<Calendario> _CalendarioList = new ObservableCollection<Calendario>();
        public ObservableCollection<Calendario> CalendarioList { get { return _CalendarioList; } set { _CalendarioList = value; OnPropertyChanged(); } }
        
        private OrganizacionManagement _CalendarioManagent = new OrganizacionManagement();
        public OrganizacionManagement CalendarioManagent { get { return _CalendarioManagent; } set { _CalendarioManagent = value; OnPropertyChanged(); } }

        public RelayCommand SetSelectionAsDiaFestivoCommand { get; private set; }
        public RelayCommand SetSelectionAsDiaLaborableCommand { get; private set; }
        public RelayCommand CreateCalendarioCommand { get; private set; }
        public RelayCommand DeleteCalendarioCommand { get; private set; }
        public RelayCommand DuplicateCalendarioCommand { get; private set; }
        public RelayCommand ResetCalendarioCommand { get; private set; }
        public RelayCommand SaveCommand { get; private set; }

        private bool OnCanSetSelectionAsDiaLaborable(object obj)
        {
            return this.SeleccionDiasFestivos.Count() > 0;
        }

        private void OnSetSelectionAsDiaLaborable(object obj)
        {
            foreach (DateTime date in this.SeleccionDiasFestivos.ToList())
            {
                if (this.DiasFestivos.Contains(date))
                    this.DiasFestivos.Remove(date);

                this.DiasLaborables.Add(date);
            }

            this.SeleccionDiasFestivos.Clear();
            this.CurrentCalendario.DiasFestivos = VectorCalendarHelper.EncodeDates(this.CurrentCalendario.DiasFestivos, this.DiasFestivos);
        }

        private bool OnCanSetSelectionAsDiaFestivo(object obj)
        {
            return this.SeleccionDiasLaborables.Count() > 0;
        }

        private void OnSetSelectionAsDiaFestivo(object obj)
        {
            foreach (DateTime date in this.SeleccionDiasLaborables.ToList())
            {
                if (this.DiasLaborables.Contains(date))
                    this.DiasLaborables.Remove(date);

                this.DiasFestivos.Add(date);
            }

            this.SeleccionDiasLaborables.Clear();
            this.CurrentCalendario.DiasFestivos = VectorCalendarHelper.EncodeDates(this.CurrentCalendario.DiasFestivos, this.DiasFestivos);
        }

        private void OnCreateCalendario(object param)
        {
            this.CurrentCalendario = this.CalendarioManagent.GetNewCalendario();
            //this.CalendarioList.Add(this.CurrentCalendario);
        }

        private bool OnCanDeleteCalendario(object obj)
        {
            return true;
        }

        private void OnDeleteCalendario(object param)
        {
            this.CalendarioManagent.DeleteCalendario(this.CurrentCalendario);
        }

        private bool OnCanDuplicateCalendario(object obj)
        {
            return this.CurrentCalendario != null;
        }

        private void OnDuplicateCalendario(object param)
        {
            this.CurrentCalendario = this.CalendarioManagent.DuplicateCalendario(this.CurrentCalendario);            
        }

        private bool OnCanResetCalendario(object obj)
        {
            return this.CurrentCalendario != null;
        }

        private void OnResetCalendario(object param)
        {
            this.CalendarioManagent.DuplicateCalendario(this.CurrentCalendario);
        }

        private bool OnCanSave(object obj)
        {
            return this.CurrentCalendario != null && !this.HasErrors;
        }

        private void OnSave(object param)
        {
            this.CalendarioManagent.UpdateCalendario(this.CurrentCalendario);

            if (!this.CalendarioList.Contains(this.CurrentCalendario))
                this.CalendarioList.Add(this.CurrentCalendario);
        }

        protected override void OnValidate(string propertyName)
        {
            //if (string.IsNullOrEmpty(propertyName) || propertyName == PROPERTY_NAME_IDENTIFICADOR)
        }


        #endregion [Subsistema de mantenimiento]

    }
}
