﻿using LiquidacionGastos.Framework;
using LiquidacionGastos.Models;
using LiquidacionGastos.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.ViewModels
{
    public class DepartamentoViewModel : ViewModel
    {
        #region [Subsistema básico]

        // Cadenas constantes de los nombres de las propiedades que se validan.
        private const string PROPERTY_NAME_NOMBRE = "Nombre";

        // Departamento actualmente en edición.
        private Departamento _CurrentDepartamento;
        public Departamento CurrentDepartamento { get { return _CurrentDepartamento; } set { _CurrentDepartamento = value; OnPropertyChanged(); } }

        // Propiedades del departamento.        
        public string Nombre { get { return this.CurrentDepartamento.Nombre; } set { this.CurrentDepartamento.Nombre = value; OnPropertyChanged(); } }

        // Propiedades del ViewModel para asignar los textos de validación
        private string _NombreValidationText;
        public string NombreValidationText { get { return _NombreValidationText; } set { _NombreValidationText = value; OnPropertyChanged(); } }

        // Usuario seleccionado actualmente en la lista de usuarios del departamento.
        private Usuario _CurrentUsuario;
        public Usuario CurrentUsuario { get { return _CurrentUsuario; } set { _CurrentUsuario = value; OnPropertyChanged(); } }
        
        // Comandos                
        public RelayCommand CancelCommand { get; private set; }
        public RelayCommand SaveCommand { get; private set; }
        public RelayCommand SelectResponsableCommand { get; private set; }
        public RelayCommand SelectSuperdepartamentoCommand { get; private set; }

        public DepartamentoViewModel()
        {
            this.ViewId = ViewName.DepartamentoView;
            this.CancelCommand = new RelayCommand(OnCancel);
            this.SaveCommand = new RelayCommand(OnSave, CanSave);
            this.SelectResponsableCommand = new RelayCommand(OnSelectResponsable);
            this.SelectSuperdepartamentoCommand = new RelayCommand(OnSelectSuperdepartamento);

            // Asignación de los campos de texto de validación para cada propiedad validada.
            this.AddValidationStringProperty("Nombre", "NombreValidationText");
        }

        private void OnCancel(object param)
        {
            this.Stop(0);
        }

        private bool CanSave(object obj)
        {
            return !this.HasErrors;
        }

        private void OnSave(object param)
        {
            this.Stop(1);
        }

        private void OnSelectResponsable(object param)
        {
            UsuarioListViewModel vm = new UsuarioListViewModel(true);
            vm.CurrentUsuario = this.CurrentDepartamento.Responsable;
            vm.Start(OnSelectResponsableCallback);
        }

        private void OnSelectResponsableCallback(ViewModel sender, ViewModelStoppedEventArgs e)
        {
            UsuarioListViewModel vm = sender as UsuarioListViewModel;
            if (e.Result != 1)
                return;

            this.CurrentDepartamento.Responsable = vm.CurrentUsuario;
            this.CurrentDepartamento.IdResponsable = vm.CurrentUsuario.IdUsuario;
        }

        private void OnSelectSuperdepartamento(object param)
        {
            DepartamentoListViewModel vm = new DepartamentoListViewModel(true);
            vm.CurrentDepartamento = this.CurrentDepartamento.Superdepartamento;
            vm.Start(OnSelectSuperdepartamentoCallback);
        }

        private void OnSelectSuperdepartamentoCallback(ViewModel sender, ViewModelStoppedEventArgs e)
        {
            DepartamentoListViewModel vm = sender as DepartamentoListViewModel;
            if (e.Result != 1)
                return;

            this.CurrentDepartamento.Superdepartamento = vm.CurrentDepartamento;            
            this.CurrentDepartamento.IdSuperdepartamento = vm.CurrentDepartamento.IdDepartamento;
        }

        #endregion [Subsistema básico]
    }
}
