﻿using LiquidacionGastos.Business;
using LiquidacionGastos.Framework;
using LiquidacionGastos.Models;
using LiquidacionGastos.Services;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace LiquidacionGastos.ViewModels
{
    public class GastoViewModel : ViewModel
    {
        #region [Subsistema básico]

        // Cadenas constantes de los nombres de las propiedades que se validan.
        private const string PROPERTY_NAME_NOMBRE = "Nombre";

        // Gasto actualmente en edición.
        private Gasto _CurrentGasto;
        public Gasto CurrentGasto { get { return _CurrentGasto; } set { _CurrentGasto = value; OnPropertyChanged(); } }

        // Propiedades del gasto.        
        public TipoGasto TipoGasto { get { return this.CurrentGasto.Tipo; } set { this.CurrentGasto.Tipo = value; OnPropertyChanged(); } }
        public DateTime Fecha { get { return this.CurrentGasto.Fecha; } set { this.CurrentGasto.Fecha = value; OnPropertyChanged(); } }
        public decimal Importe { get { return this.CurrentGasto.Importe; } set { this.CurrentGasto.Importe = value; OnPropertyChanged(); } }
        public CambioMoneda CambioMoneda { get { return this.CurrentGasto.CambioMoneda; } set { this.CurrentGasto.CambioMoneda = value; OnPropertyChanged(); } }
        public string Descripcion { get { return this.CurrentGasto.Descripcion; } set { this.CurrentGasto.Descripcion = value; OnPropertyChanged(); } }
                
        private ObservableCollection<TipoGasto> _TipoGastoList = new ObservableCollection<TipoGasto>();
        public ObservableCollection<TipoGasto> TipoGastoList { get { return _TipoGastoList; } set { _TipoGastoList = value; OnPropertyChanged(); } }

        private ObservableCollection<CambioMoneda> _CambioMonedaList = new ObservableCollection<CambioMoneda>();
        public ObservableCollection<CambioMoneda> CambioMonedaList { get { return _CambioMonedaList; } set { _CambioMonedaList = value; OnPropertyChanged(); } }
        
        // Comandos
        public RelayCommand CancelCommand { get; set; }
        public RelayCommand SaveCommand { get; set; }

        private LiquidacionManagement _GastoManagent = new LiquidacionManagement();
        public LiquidacionManagement GastoManagent { get { return _GastoManagent; } set { _GastoManagent = value; OnPropertyChanged(); } }
                    
        public GastoViewModel()
        {
            this.ViewId = ViewName.GastoView;
            this.TipoGastoList.AddRange(this.GastoManagent.ListTiposGasto());
            this.CambioMonedaList.AddRange(this.GastoManagent.ListCambiosMonedaMostRecent());            

            this.CancelCommand = new RelayCommand(OnCancel);
            this.SaveCommand = new RelayCommand(OnSave, CanSave);

            this.AddImageCommand = new RelayCommand(OnAddImage);
            this.RemoveImageCommand = new RelayCommand(OnRemoveImage);

            // Asignación de los campos de texto de validación para cada propiedad validada.
            //this.AddValidationStringProperty("Fecha", "FechaValidationText");
        }

        private void OnCancel(object param)
        {
            this.Stop(0);
        }

        private bool CanSave(object obj)
        {
            return !this.HasErrors;
        }

        private void OnSave(object param)
        {
            this.Stop(1);
        }

        protected override void OnValidate(string propertyName)
        {
            //if (string.IsNullOrEmpty(propertyName) || propertyName == PROPERTY_NAME_NOMBRE)
            //    OnValidate_Property();
        }

        #endregion [Subsistema básico]

        #region [Subsistema edición de imagen]

        public BitmapImage ImagenComprobante { get { return ImageFromBuffer(this.CurrentGasto.ImagenComprobante); } set { this.CurrentGasto.ImagenComprobante = BufferFromImage(value); OnPropertyChanged(); } }
                    
        // Comandos
        public RelayCommand AddImageCommand { get; set; }
        public RelayCommand RemoveImageCommand { get; set; }

        private void OnRemoveImage(object obj)
        {
            this.ImagenComprobante = null;
        }

        private void OnAddImage(object obj)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            if (ofd.ShowDialog() != true)
                return;

            Stream reader = File.OpenRead(ofd.FileName);
            BitmapImage imgsrc = new BitmapImage();
            imgsrc.BeginInit();
            imgsrc.StreamSource = reader;
            imgsrc.EndInit();

            this.ImagenComprobante = imgsrc;
        }

        private BitmapImage ImageFromBuffer(byte[] bytes)
        {
            if (bytes == null)
                return null;

            MemoryStream stream = new MemoryStream(bytes);
            BitmapImage image = new BitmapImage();
            image.BeginInit();
            image.StreamSource = stream;
            image.EndInit();
            return image;
        }

        private byte[] BufferFromImage(BitmapImage imageSource)
        {
            if (imageSource == null)
                return null;

            Stream stream = imageSource.StreamSource;
            stream.Position = 0;

            using (MemoryStream ms = new MemoryStream())
            {
                stream.CopyTo(ms);
                return ms.ToArray();
            }            
        }

        #endregion [Subsistema edición de imagen]
    }
}

