﻿using LiquidacionGastos.Framework;
using LiquidacionGastos.Services;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;

namespace LiquidacionGastos.ViewModels
{
    public class LoginViewModel : ViewModel //, INotifyDataErrorInfo
    {
        private string _UserId;
        public string UserId { get { return _UserId; } set { _UserId = value; OnPropertyChanged();  } }

        private string _Password;
        public string Password { get { return _Password; } set { _Password = value; OnPropertyChanged(); } }

        private bool _IsActiveProgressRing;
        public bool IsActiveProgressRing { get { return _IsActiveProgressRing; } set { _IsActiveProgressRing = value; OnPropertyChanged(); } }

        private Visibility _WarningLabelVisibility;
        public Visibility WarningLabelVisibility { get { return _WarningLabelVisibility; } set { _WarningLabelVisibility = value; OnPropertyChanged(); } }

        private bool _IsEnabledLoginEdition;
        public bool IsEnabledLoginEdition { get { return _IsEnabledLoginEdition; } set { _IsEnabledLoginEdition = value; OnPropertyChanged(); } }

        public RelayCommand LoginCommand { get; set; }

        public LoginViewModel()
        {
            this.ViewId = ViewName.LoginView;
            this.UserId = string.Empty;
            this.IsActiveProgressRing = false;
            this.WarningLabelVisibility = Visibility.Hidden;
            this.IsEnabledLoginEdition = true;
            this.LoginCommand = new RelayCommand(OnLogin);
        }

        private async void OnLogin(object param)
        {
            this.IsEnabledLoginEdition = false;
            this.WarningLabelVisibility = Visibility.Hidden;
            this.IsActiveProgressRing = true;            

            var parameter = param as PasswordBox;
            this.Password = parameter.Password;

            bool resultOk = await AuthenticateUserAsync(this.UserId, this.Password);

            this.IsActiveProgressRing = false;

            if (resultOk)
            {
                this.Stop();
                return;
            }
            else
            {
                this.WarningLabelVisibility = Visibility.Visible;
                this.IsEnabledLoginEdition = true;
            }
        }

        private Task<bool> AuthenticateUserAsync(string userid, string passwd)
        {
            return Task.Run<bool>(() =>
            {
                IAuthenticationService service = ServiceLocator.Resolve<IAuthenticationService>();
                return service.AuthenticateUser(this.UserId, this.Password);
            });
        }


        #region INotifyDataErrorInfo

        //public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;

        //public System.Collections.IEnumerable GetErrors(string propertyName)
        //{
        //    List<string> list = new List<string>();
        //    if (propertyName == "UserId")
        //    {
        //        if (this.UserId != "alberto@dnc.com" && this.UserId != "root" )
        //        {
        //            list.Add("Identificador de usuario NO VALIDO");
        //            return list;
        //        }                    
        //    }

        //return null;
        //}

        //public bool HasErrors
        //{
        //    get { return this.UserId != "alberto@dnc.com" && this.UserId != "root"; }
        //}

        #endregion INotifyDataErrorInfo
    }
}
