﻿using LiquidacionGastos.Framework;
using LiquidacionGastos.Models;
using LiquidacionGastos.Security;
using LiquidacionGastos.Services;
using LiquidacionGastos.Synch;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Resources;
using System.Security.Principal;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;

namespace LiquidacionGastos.ViewModels
{
    public enum MainWindowOperationState
    {
        SinOperacion = 0,
        Login,
        Liquidacion,
        Autorizacion,
        Departamento,
        Usuario,
        Calendario,
        Opciones
    };

    public class MainWindowViewModel : ViewModel
    {
        // Impresión: http://www.nullskull.com/a/1378/wpf-printing-and-print-preview.aspx

        public RelayCommand CloseCommand { get; private set; }
        public RelayCommand LogOutCommand { get; private set; }
        public RelayCommand ChangeConectionStatusCommand { get; private set; }
        public RelayCommand ChangeReconnectionStatusCommand { get; private set; }
        public RelayCommand ShowLiquidacionesCommand { get; private set; }
        public RelayCommand ShowAutorizacionesCommand { get; private set; }                        
        public RelayCommand ShowDepartamentosCommand { get; private set; }                        
        public RelayCommand ShowUsuariosCommand { get; private set; }                        
        public RelayCommand ShowCalendariosCommand { get; private set; }                        
        public RelayCommand ShowOpcionesCommand { get; private set; }                        

        private bool _IsLoggedIn;
        public bool IsLoggedIn { get { return _IsLoggedIn; } private set { _IsLoggedIn = value; OnPropertyChanged("IsLoggedIn"); } }

        private bool _IsDiconnected;
        public bool IsDisconnected 
        { 
            get {return _IsDiconnected; } 
            private set 
            {   _IsDiconnected = value;
                 OnPropertyChanged();
            } 
        }
    
        private bool _IsReconnectionEnabled;
        public bool IsReconnectionEnabled { get { return _IsReconnectionEnabled; } private set { _IsReconnectionEnabled = value; OnPropertyChanged(); } }

        public string Title { get; set; }

        private StateCommand<MainWindowOperationState> MainWindowState { get; set; }
                
        private Usuario _CurrentUsuario;
        public Usuario CurrentUsuario { get { return _CurrentUsuario; } set { _CurrentUsuario = value; OnPropertyChanged(); } }        

        private Visibility _LiquidacionesMenuVisibility;
        public Visibility LiquidacionesMenuVisibility { get { return _LiquidacionesMenuVisibility; } set { _LiquidacionesMenuVisibility = value; OnPropertyChanged(); } }

        private Visibility _AutorizacionesMenuVisibility;
        public Visibility AutorizacionesMenuVisibility { get { return _AutorizacionesMenuVisibility; } set { _AutorizacionesMenuVisibility = value; OnPropertyChanged(); } }

        private Visibility _OrganizacionMenuVisibility;
        public Visibility OrganizacionMenuVisibility { get { return _OrganizacionMenuVisibility; } set { _OrganizacionMenuVisibility = value; OnPropertyChanged(); } }

        private Visibility _OpcionesMenuVisibility;
        public Visibility OpcionesMenuVisibility { get { return _OpcionesMenuVisibility; } set { _OpcionesMenuVisibility = value; OnPropertyChanged(); } }
        
        private bool _IsLiquidacionesMenuEnabled;
        public bool IsLiquidacionesMenuEnabled { get { return _IsLiquidacionesMenuEnabled; } set { _IsLiquidacionesMenuEnabled = value; OnPropertyChanged(); } }

        private bool _IsAutorizacionesMenuEnabled;
        public bool IsAutorizacionesMenuEnabled { get { return _IsAutorizacionesMenuEnabled; } set { _IsAutorizacionesMenuEnabled = value; OnPropertyChanged(); } }

        private bool _IsDepartamentosMenuEnabled;
        public bool IsDepartamentosMenuEnabled { get { return _IsDepartamentosMenuEnabled; } set { _IsDepartamentosMenuEnabled = value; OnPropertyChanged(); } }

        private bool _IsUsuariosMenuEnabled;
        public bool IsUsuariosMenuEnabled { get { return _IsUsuariosMenuEnabled; } set { _IsUsuariosMenuEnabled = value; OnPropertyChanged(); } }

        private bool _IsCalendariosMenuEnabled;
        public bool IsCalendariosMenuEnabled { get { return _IsCalendariosMenuEnabled; } set { _IsCalendariosMenuEnabled = value; OnPropertyChanged(); } }

        private ViewModel _CurrentViewModel;
        private ViewModel CurrentViewModel
        {
            get
            {
                return _CurrentViewModel;
            }
            set
            {
                if (_CurrentViewModel != null)
                {
                    _CurrentViewModel.Stop();
                }

                _CurrentViewModel = value;

                if (_CurrentViewModel != null)
                {
                    _CurrentViewModel.Stopped += OnCurrentViewModelStopped;
                    _CurrentViewModel.Start();

                }
            }
        }

        private Synchronizer _Synchronizer;
        public Synchronizer Synchronizer 
        { 
            get { return _Synchronizer; } 
            set 
            {
                if (value == null && _Synchronizer != null)
                {
                    _Synchronizer.ServerNotReached -= OnServerNotReached;
                    _Synchronizer.SynchronizationCompleted -= OnSynchronizationCompleted;
                }

                
                _Synchronizer = value;

                if (_Synchronizer != null)
                {
                    _Synchronizer.ServerNotReached += OnServerNotReached;
                    _Synchronizer.SynchronizationCompleted += OnSynchronizationCompleted;
                }
            } 
        }

        private void OnServerConnect()
        {
            this.IsDisconnected = false;

            if (this.Synchronizer == null)
                return;
            
            this.IsDisconnected = !this.Synchronizer.Start();
        }

        private void OnServerDisconnect()
        {
            this.IsDisconnected = true;

            if (this.Synchronizer == null)
                return;

            this.Synchronizer.Stop();
        }
        
        private void OnServerNotReached(object sender, EventArgs e)
        {
            this.IsDisconnected = true;
            ToastNotification.Show("No se ha podido conectar con el servidor. Modo desconectado activado.");
        }

        private void OnSynchronizationCompleted(object sender, EventArgs e)
        {
            ToastNotification.Show("Sincronización completada");
        }

        private void OnCurrentViewModelStopped(object sender, EventArgs e)
        {
            ViewModel vm = sender as ViewModel;
            vm.Stopped -= OnCurrentViewModelStopped;
            this.MainWindowState.CurrentStateId = MainWindowOperationState.SinOperacion;
        }

        public MainWindowViewModel()
        {
            this.Title = LiquidacionGastos.Properties.Resources.ApplicationTitle;

            this.IsLoggedIn = false;
            this.IsDisconnected = true;
            this.IsReconnectionEnabled = false;
           
            this.MainWindowState = new StateCommand<MainWindowOperationState>();

            this.MainWindowState.AddState(MainWindowOperationState.Login, 
                                          OnEnterLoginState, 
                                          new MainWindowOperationState[] {MainWindowOperationState.SinOperacion});

            this.MainWindowState.AddState(MainWindowOperationState.SinOperacion, 
                                          OnEnterSinOperacionState, 
                                          new MainWindowOperationState[] {MainWindowOperationState.Login,
                                                                          MainWindowOperationState.Liquidacion,
                                                                          MainWindowOperationState.Autorizacion,
                                                                          MainWindowOperationState.Departamento,
                                                                          MainWindowOperationState.Usuario,
                                                                          MainWindowOperationState.Calendario,
                                                                          MainWindowOperationState.Opciones});

            this.MainWindowState.AddState(MainWindowOperationState.Liquidacion,
                                          OnEnterLiquidacionState,
                                          new MainWindowOperationState[] {MainWindowOperationState.SinOperacion});

            this.MainWindowState.AddState(MainWindowOperationState.Autorizacion,
                                          OnEnterAutorizacionState,
                                          new MainWindowOperationState[] { MainWindowOperationState.SinOperacion });

            this.MainWindowState.AddState(MainWindowOperationState.Departamento,
                                          OnEnterDepartamentoState,
                                          new MainWindowOperationState[] { MainWindowOperationState.SinOperacion });

            this.MainWindowState.AddState(MainWindowOperationState.Usuario,
                                          OnEnterUsuarioState,
                                          new MainWindowOperationState[] { MainWindowOperationState.SinOperacion });

            this.MainWindowState.AddState(MainWindowOperationState.Calendario,
                                          OnEnterCalendarioState,
                                          new MainWindowOperationState[] { MainWindowOperationState.SinOperacion });

            this.MainWindowState.AddState(MainWindowOperationState.Opciones,
                                          OnEnterOpcionesState,
                                          new MainWindowOperationState[] { MainWindowOperationState.SinOperacion });

            this.Synchronizer = ServiceLocator.Resolve<Synchronizer>();

            this.ChangeConectionStatusCommand = new RelayCommand(OnChangeConectionStatus);
            this.ChangeReconnectionStatusCommand = new RelayCommand(OnChangeReconnectionStatus);
            this.LogOutCommand = new RelayCommand(OnLogout);            
            this.CloseCommand = new RelayCommand(OnRequestClose);            
            this.ShowLiquidacionesCommand = new RelayCommand(OnShowLiquidaciones);
            this.ShowAutorizacionesCommand = new RelayCommand(OnShowAutorizaciones);
            this.ShowDepartamentosCommand = new RelayCommand(OnShowDepartamentos);
            this.ShowUsuariosCommand = new RelayCommand(OnShowUsuarios);
            this.ShowCalendariosCommand = new RelayCommand(OnShowCalendarios);
            this.ShowOpcionesCommand = new RelayCommand(OnShowOpciones);
        }

        private void SetupMenu(StateCommand<MainWindowOperationState> stateCommand)
        { 
            // Por defecto todos las opciones no están disponibles.
            this.LiquidacionesMenuVisibility = Visibility.Collapsed;
            this.AutorizacionesMenuVisibility = Visibility.Collapsed;
            this.OrganizacionMenuVisibility = Visibility.Collapsed;
            this.OpcionesMenuVisibility = Visibility.Collapsed;

            // Si no hay un usuario logeado, salir.
            if (this.CurrentUsuario == null)
                return;

            // Obtener el objeto Principal del usuario actual registrado.
            IUsuarioPrincipal usuarioPrincipal = GenericPrincipal.Current as IUsuarioPrincipal;

            // Opciones para todos los usuarios.    
            this.OpcionesMenuVisibility = stateCommand.IsValidTransition(MainWindowOperationState.Opciones) ? Visibility.Visible : Visibility.Collapsed;
            
            // Opciones para usuario con rol Usuario.
            if (usuarioPrincipal.IsInRole(UsuarioRoles.Usuario.ToString()))
            {
                this.LiquidacionesMenuVisibility = stateCommand.IsValidTransition(MainWindowOperationState.Liquidacion) ? Visibility.Visible : Visibility.Collapsed;
                this.IsLiquidacionesMenuEnabled = stateCommand.IsValidTransition(MainWindowOperationState.Liquidacion);
            }
            
            // Opcines para usuario con rol Autorizador.
            if (usuarioPrincipal.IsInRole(UsuarioRoles.Autorizador.ToString()))
            {
                this.AutorizacionesMenuVisibility = stateCommand.IsValidTransition(MainWindowOperationState.Autorizacion) ? Visibility.Visible : Visibility.Collapsed;
                this.IsAutorizacionesMenuEnabled = stateCommand.IsValidTransition(MainWindowOperationState.Autorizacion);
            }

            // Opcines para usuario con rol Administrador.
            if (usuarioPrincipal.IsInRole(UsuarioRoles.Administrador.ToString()))
            {
                this.OrganizacionMenuVisibility = stateCommand.IsValidTransition(MainWindowOperationState.Calendario) ? Visibility.Visible : Visibility.Collapsed;
                this.IsDepartamentosMenuEnabled = stateCommand.IsValidTransition(MainWindowOperationState.Departamento);
                this.IsUsuariosMenuEnabled = stateCommand.IsValidTransition(MainWindowOperationState.Usuario);
                this.IsCalendariosMenuEnabled = stateCommand.IsValidTransition(MainWindowOperationState.Calendario);
            }
        }

        private void OnEnterLoginState(StateCommand<MainWindowOperationState> stateCommand)
        {
            this.SetupMenu(stateCommand);

            // Cerrar todos los procesos abiertos por el usuario.
            this.IsLoggedIn = false;

            this.CurrentUsuario = null;

            LoginViewModel vm = new LoginViewModel();            
            vm.Stopped += OnLogginViewModelStopped;
            this.CurrentViewModel = vm;            
        }

        private void OnLogginViewModelStopped(object sender, EventArgs e)
        {
            LoginViewModel vm = sender as LoginViewModel;
            vm.Stopped -= OnLogginViewModelStopped;
            
            // Comprobar si el usuario está logeado.
            this.IsLoggedIn = true;
        }

        private void OnEnterSinOperacionState(StateCommand<MainWindowOperationState> stateCommand)
        {            
            if ((GenericPrincipal.Current as IUsuarioPrincipal)!= null)
                this.CurrentUsuario = ((IUsuarioPrincipal)GenericPrincipal.Current).Usuario;
            else
                this.CurrentUsuario = null;

            this.SetupMenu(stateCommand);
            this.CurrentViewModel = null;
        }

        private void OnEnterLiquidacionState(StateCommand<MainWindowOperationState> stateCommand)
        {
            this.SetupMenu(stateCommand);
            LiquidacionListViewModel vm = new LiquidacionListViewModel(false);
            this.CurrentViewModel = vm;
        }

        private void OnEnterAutorizacionState(StateCommand<MainWindowOperationState> stateCommand)
        {
            this.SetupMenu(stateCommand);
            LiquidacionListViewModel vm = new LiquidacionListViewModel(true);
            vm.IsAuthorizationMode = true;
            this.CurrentViewModel = vm;
        }

        private void OnEnterDepartamentoState(StateCommand<MainWindowOperationState> stateCommand)
        {
            this.SetupMenu(stateCommand);
            DepartamentoListViewModel vm = new DepartamentoListViewModel(false);
            this.CurrentViewModel = vm;
        }

        private void OnEnterUsuarioState(StateCommand<MainWindowOperationState> stateCommand)
        {
            this.SetupMenu(stateCommand);
            UsuarioListViewModel vm = new UsuarioListViewModel(false);
            this.CurrentViewModel = vm;
        }

        private void OnEnterCalendarioState(StateCommand<MainWindowOperationState> stateCommand)
        {
            this.SetupMenu(stateCommand);
            CalendariosViewModel vm = new CalendariosViewModel();
            this.CurrentViewModel = vm;
        }

        private void OnEnterOpcionesState(StateCommand<MainWindowOperationState> stateCommand)
        {
            this.SetupMenu(stateCommand);
            OpcionesViewModel vm = new OpcionesViewModel();
            this.CurrentViewModel = vm;
        }

        private void OnShowLiquidaciones(object param)
        {
            this.MainWindowState.CurrentStateId = MainWindowOperationState.Liquidacion;
        }
    
        private void OnLogout(object param)
        {
            this.MainWindowState.CurrentStateId = MainWindowOperationState.Login;
        }

        private void OnRequestClose(object param)
        {
            this.Stop(0);
        }

        protected override void OnStop()
        {
            //base.OnStop();            
            this.Synchronizer.Stop();
            ApplicationServices.Application.Shutdown();            
        }

        private void OnChangeConectionStatus(object param) 
        {
            if (this.IsDisconnected)
                this.OnServerConnect();
            else
                this.OnServerDisconnect();

            ToastNotification.Show("Trabajando en modo " + (this.IsDisconnected ? "desconectado" : "conectado") + ".");            
        }

        private void OnChangeReconnectionStatus(object param) 
        {
            this.IsReconnectionEnabled = !this.IsReconnectionEnabled;
            ToastNotification.Show("Reconexión " + (this.IsReconnectionEnabled ? "activada" : "desactivada") + ".");
        }

        private void OnEdit(object param)
        {
        }

        private void Confirmar()
        {
            ConfirmacionViewModel c = new ConfirmacionViewModel();
            c.Confirmar(this.ConfirmarCallback);
        }

        private void ConfirmarCallback(ConfirmacionViewModel param)
        {
            MessageBox.Show("Confirmacion");
        }


        //////////////////////

        public void Login()
        {
            this.OnLogout(null);
        }

        private void OnShowAutorizaciones(object param)
        {
            this.MainWindowState.CurrentStateId = MainWindowOperationState.Autorizacion;
        }

        private void OnShowDepartamentos(object param)
        {
            this.MainWindowState.CurrentStateId = MainWindowOperationState.Departamento;
        }

        private void OnShowUsuarios(object param)
        {
            this.MainWindowState.CurrentStateId = MainWindowOperationState.Usuario;
        }

        private void OnShowCalendarios(object param)
        {
            this.MainWindowState.CurrentStateId = MainWindowOperationState.Calendario;
        }

        private void OnShowOpciones(object param)
        {
            this.MainWindowState.CurrentStateId = MainWindowOperationState.Opciones;
        }

    }
}
