﻿using LiquidacionGastos.Framework;
using LiquidacionGastos.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace LiquidacionGastos.ViewModels
{
    public class MessageDialogViewModel : ViewModel
    {
        protected Window window;

        public RelayCommand OkCommand { get; set; }
        public RelayCommand CancelCommand { get; set; }

        public MessageDialogViewModel()
        {
            this.ViewId = ViewName.ConfirmacionView;
            this.OkCommand = new RelayCommand(OnAcceptCommand);
            this.CancelCommand = new RelayCommand(OnCancelCommand);
        }

        private void OnAcceptCommand(object param)
        {
            this.Stop(1);
        }

        private void OnCancelCommand(object param)
        {
            this.Stop(0);
        }

        protected override void OnStart()
        {
            //base.OnStart();
            OnRegisterView();

            UserControl view = ApplicationServices.NavigationService.GetView(this, this.ViewId);

            window = new Window();
            window.WindowStyle = WindowStyle.None;
            window.ResizeMode = ResizeMode.NoResize;
            Window mainWindow = ApplicationServices.Application.MainWindow;
            window.Width = ((Grid)mainWindow.Content).RenderSize.Width;
            window.Height = ((Grid)mainWindow.Content).RenderSize.Height/3;
            window.Owner = mainWindow;
            window.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;
            window.ShowInTaskbar = false;
            window.Background = new SolidColorBrush(Colors.LightBlue);            
            //window.Background = new SolidColorBrush(Color.FromArgb(0xff, 0x8, 0x6f, 0x9e));            

            view.Width = double.NaN;
            view.Height = double.NaN;
            view.VerticalContentAlignment = VerticalAlignment.Center;
            view.HorizontalContentAlignment = HorizontalAlignment.Center;
            window.Content = view;

            window.Opacity = 0.0;

            EventHandler handler = null;
            handler = delegate
            {
                window.Activated -= handler;
                FadeIn(window);
            };
            window.Activated += handler;
            window.ShowDialog();           
        }

        protected override void OnStop()
        {
            FadeOut(window);            
            OnUnregisterView();
            //base.OnStop();
        }

        private void FadeIn(Window window)
        {
            Fade(window, 1.0, 0.5, null);
        }

        private void FadeOut(Window window)
        {
            Fade(window, 0.0, 0.5, delegate { window.Close(); });
        }

        private DoubleAnimation da;
        private void Fade(Window window, double fadeTo, double seconds, EventHandler ev)
        {
            da = new DoubleAnimation();
            da.To = fadeTo;
            da.By = 1.0;
            da.Duration = new Duration(TimeSpan.FromSeconds(seconds));

            if (ev != null)
                da.Completed += ev;

            window.BeginAnimation(Window.OpacityProperty, da);
        }

    }
}
