﻿using LiquidacionGastos.Models;
using LiquidacionGastos.Repositories;
using LiquidacionGastos.UnitsOfWork;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.WebServiceRepositories
{
    public class CompleteSyncUnitOfWork : WebServicesUnitOfWork, IDisposable
    {
        private TimeSyncWebServiceRepository TimeSyncRepositoryRemote { get; set; }
        private AuthenticationWebServiceRepository AuthenticationRepositoryRemote { get; set; }
        private TrackingEntryWebServiceRepository TrackingRepositoryRemote { get; set; }
        private EntitySyncWebServiceRepository EntitySyncRepositoryRemote { get; set; }
        
        private LGContext Context { get; set; }
        private EntityFrameworkUnitOfWork InnerUnitOfWork { get; set; }
        private TrackingEntryRepository TrackingRepositoryLocal { get; set; }        
        private CalendarioRepository CalendarioRepositoryLocal { get; set; }
        private DepartamentoRepository DepartamentoRepositoryLocal { get; set; }
        private MonedaRepository MonedaRepositoryLocal { get; set; }
        private TipoGastoRepository TipoGastoRepositoryLocal { get; set; }
        private EstadoLiquidacionRepository EstadoLiquidacionRepositoryLocal { get; set; }
        private UsuarioRepository UsuarioRepositoryLocal { get; set; }
        private CambioMonedaRepository CambioMonedaRepositoryLocal { get; set; }
        private LiquidacionRepository LiquidacionRepositoryLocal { get; set; }
        private GastoRepository GastoRepositoryLocal { get; set; }
        private CambioEstadoLiquidacionRepository CambioEstadoLiquidacionRepositoryLocal { get; set; }
        private NotificacionRepository NotificacionRepositoryLocal { get; set; }
               
        public CompleteSyncUnitOfWork()
        {
        }

        public void Commit()
        {
            this.InnerUnitOfWork.Commit();
        }

        public void Dispose()
        {
            this.InnerUnitOfWork.Dispose();
        }

        protected override void OnInitialize()
        {
            base.OnInitialize();

            this.Context = new LGContext();
            this.InnerUnitOfWork = new EntityFrameworkUnitOfWork(this.Context);
            this.TrackingRepositoryLocal = new TrackingEntryRepository(this.Context);            
            this.CalendarioRepositoryLocal = new CalendarioRepository(this.Context);
            this.DepartamentoRepositoryLocal = new DepartamentoRepository(this.Context);
            this.MonedaRepositoryLocal = new MonedaRepository(this.Context);
            this.TipoGastoRepositoryLocal = new TipoGastoRepository(this.Context);
            this.EstadoLiquidacionRepositoryLocal = new EstadoLiquidacionRepository(this.Context);
            this.UsuarioRepositoryLocal = new UsuarioRepository(this.Context);
            this.CambioMonedaRepositoryLocal = new CambioMonedaRepository(this.Context);
            this.LiquidacionRepositoryLocal = new LiquidacionRepository(this.Context);
            this.GastoRepositoryLocal = new GastoRepository(this.Context);
            this.CambioEstadoLiquidacionRepositoryLocal = new CambioEstadoLiquidacionRepository(this.Context);
            this.NotificacionRepositoryLocal = new NotificacionRepository(this.Context);

            // Desactivar tracking de los respositorios locales.
            this.CalendarioRepositoryLocal.IsTrackingEnabled = false;
            this.DepartamentoRepositoryLocal.IsTrackingEnabled = false;
            this.MonedaRepositoryLocal.IsTrackingEnabled = false;
            this.TipoGastoRepositoryLocal.IsTrackingEnabled = false;
            this.EstadoLiquidacionRepositoryLocal.IsTrackingEnabled = false;
            this.UsuarioRepositoryLocal.IsTrackingEnabled = false;
            this.CambioMonedaRepositoryLocal.IsTrackingEnabled = false;
            this.LiquidacionRepositoryLocal.IsTrackingEnabled = false;
            this.GastoRepositoryLocal.IsTrackingEnabled = false;
            this.CambioEstadoLiquidacionRepositoryLocal.IsTrackingEnabled = false;
            this.NotificacionRepositoryLocal.IsTrackingEnabled = false;
        }

        protected override void OnInitializeWebServices()
        {
            this.AuthenticationRepositoryRemote = new AuthenticationWebServiceRepository();
            this.TimeSyncRepositoryRemote = new TimeSyncWebServiceRepository();
            this.TrackingRepositoryRemote = new TrackingEntryWebServiceRepository();
            this.EntitySyncRepositoryRemote = new EntitySyncWebServiceRepository();
        }

        public long Syncronize()
        {
            long entitySynchronizedCount = 0;

            Debug.WriteLine("Cliente: Iniciando sincronización completa");

            // Obtener los datos de seguimiento de cambio de entidades del servidor y local.
            DateTime dateFrom = DateTime.Today.Subtract(TimeSpan.FromDays(1));
            IList<TrackingEntry> serverTrackingEntryList = this.TrackingRepositoryRemote.ListTrackingEntries(dateFrom);
//#if DEBUG
//            // Probar solo con una entidad.
//            TrackingEntry ts1 = serverTrackingEntryList[0];
//            serverTrackingEntryList.Clear();
//            serverTrackingEntryList.Add(ts1);
//#endif

            IList<TrackingEntry> localTrackingEntryList = this.ListLocalTrackingEntries(dateFrom);

//#if DEBUG
//            // Probar solo con una entidad.
//            if (localTrackingEntryList.Count() > 0)
//            {
//                TrackingEntry tl1 = localTrackingEntryList[0];
//                localTrackingEntryList.Clear();
//                localTrackingEntryList.Add(tl1);
//            }
//#endif


            // Crear un diccionario con la clave de cada entrada para la lista local y del servidor.
            IDictionary<Guid, TrackingEntry> serverTrackingEntryDic = new Dictionary<Guid, TrackingEntry>();
            
            foreach (TrackingEntry entry in serverTrackingEntryList)
                serverTrackingEntryDic.Add(entry.IdTracking, entry);

            IDictionary<Guid, TrackingEntry> localTrackingEntryDic = new Dictionary<Guid, TrackingEntry>();

            foreach (TrackingEntry entry in localTrackingEntryList)
                localTrackingEntryDic.Add(entry.IdTracking, entry);
            
            // Indizar las entradas por operacion y destino.
            IList<TrackingEntry> serverInsertEntries = new List<TrackingEntry>();
            IList<TrackingEntry> localInsertEntries = new List<TrackingEntry>();
            IList<TrackingEntry> serverUpdateEntries = new List<TrackingEntry>();
            IList<TrackingEntry> localUpdateEntries = new List<TrackingEntry>();
            IList<TrackingEntry> serverDeleteEntries = new List<TrackingEntry>();
            IList<TrackingEntry> localDeleteEntries = new List<TrackingEntry>();

            // Crear un dicionario para las entradas descartadas
            IDictionary<Guid, TrackingEntry> discardedTrackingEntryDic = new Dictionary<Guid, TrackingEntry>();

            // Comprobar que entradas del servidor hay que copiar en local.
            foreach (TrackingEntry entry in serverTrackingEntryList)
            {
                // Comprobar si las entradas están en local.
                TrackingEntry localTrackingEntry;
                localTrackingEntryDic.TryGetValue(entry.IdTracking, out localTrackingEntry);

                // Si lo está verificar si la fecha del servidor es igual que la local. Si no lo está NO excluir la copia.
                bool exclude = localTrackingEntry != null ? (entry.Fecha == localTrackingEntry.Fecha) : false;

                // Si las fechas son iguales descartar las dos entradas (tienen la misma clave) y pasar a la siguiente.
                if (exclude)
                {
                    discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                    continue;     
                }
                    
                // Si la entrada NO está en local ó la fecha de la entrada del servidor es más actual copiar en local.
                bool copyToLocal = localTrackingEntry != null ? (entry.Fecha > localTrackingEntry.Fecha) : true;

                // Si no hay que copiarlo en local, entonces desechar las dos entradas (tienen la misma clave) y pasar a la siguiente.
                if (!copyToLocal)
                {
                    discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                    continue;
                }

                // Si hemos llegado aquí, la entrada se debe copiar en local. Clasificar la operación:
                switch (entry.Operacion)
                {
                    case (int)TrackingEntry.OperacionType.Insercion:
                        // Registar la operación de copia en local.
                        localInsertEntries.Add(entry);
                        // Desechar la entrada para no volverla a comparar en el otro sentido (local -> servidor)
                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                        break;

                    case (int)TrackingEntry.OperacionType.Actualizacion:
                        // Registar la operación de actualización en local.
                        localUpdateEntries.Add(entry);
                        // Desechar la entrada para no volverla a comparar en el otro sentido (local -> servidor)
                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                        break;

                    case (int)TrackingEntry.OperacionType.Borrado:
                        // Registar la operación de eliminación en local.
                        localDeleteEntries.Add(entry);
                        // Desechar la entrada para no volverla a comparar en el otro sentido (local -> servidor)
                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                        break;
                }
            }

            // Comprobar que entradas de local hay que copiar en el servidor.
            foreach (TrackingEntry entry in localTrackingEntryList)
            {

                // Comprobar si la entrada ya fue descartada. Si ya fue descartada, pasar a la siguiente entrada de local.
                if (discardedTrackingEntryDic.ContainsKey(entry.IdTracking))
                    continue;

                // Si la entrada no esta en la lista de entradas descartadas entonces hay que copiarla al servidor,
                // ya que es el único caso que resta por tratar.

                // Si hemos llegado aquí, la entrada se debe copiar en servidor. Clasificar la operación:
                switch (entry.Operacion)
                {
                    case (int)TrackingEntry.OperacionType.Insercion:
                        // Registar la operación de copia en servidor.
                        serverInsertEntries.Add(entry);
                        break;

                    case (int)TrackingEntry.OperacionType.Actualizacion:
                        // Registar la operación de actualización en servidor.
                        serverUpdateEntries.Add(entry);
                        break;

                    case (int)TrackingEntry.OperacionType.Borrado:
                        // Registar la operación de eliminación en servidor.
                        serverDeleteEntries.Add(entry);
                        break;
                }
            }

            // Vaciar el diccionario de entradas descartadas para utilizarlo en la progresión de copia
            // en las siguientes fases.
            discardedTrackingEntryDic.Clear();
            
            for (int iteration = 1; iteration < 3; iteration++)
            {
                #region [Sincronización por fases]

                for (int stage = 1; stage < 31; stage++)
                {
                    #region [Gestión de etapas de sincronización]

                    IList<TrackingEntry> currentEntryList = null;

                    // Seleccionar la lista de entradas que se deben procesar.
                    if (stage <= 5)
                    {
                        currentEntryList = localInsertEntries;
                    }
                    else if (stage <= 10)
                    {
                        currentEntryList = localUpdateEntries;
                    }
                    else if (stage <= 15)
                    {
                        currentEntryList = localDeleteEntries;
                    }
                    else if (stage <= 20)
                    {
                        if (stage == 16)
                        {
                            // Guardar los cambios en base de datos.
                            //globalUnitOfWork.Commit();

                            // Vaciar el diccionario de entradas descartadas para utilizarlo en la progresión 
                            // de copia hacia el servidoren las siguientes fases.

                            discardedTrackingEntryDic.Clear();
                        }

                        currentEntryList = serverInsertEntries;
                    }
                    else if (stage <= 25)
                    {
                        currentEntryList = serverUpdateEntries;
                    }
                    else if (stage <= 30)
                    {
                        currentEntryList = serverDeleteEntries;
                    }

                    #endregion [Gestión de etapas de sincronización]

                    foreach (TrackingEntry entry in currentEntryList)
                    {
                        // Si la entrada ya fue descartada, pasar a la siguiente entrada.
                        if (discardedTrackingEntryDic.ContainsKey(entry.IdTracking))
                            continue;

                        // Incrementar el contador de entidades sincronizadas.
                        entitySynchronizedCount++;

                        Debug.WriteLine(string.Format("Ejecutanto operación de {0} para {1} con ID {2}", ((TrackingEntry.OperacionType)entry.Operacion).ToString(), ((TrackingEntry.EntidadType)entry.Entidad).ToString(), entry.Id == null ? entry.Guid.ToString() : entry.Id.ToString()));

                        switch (stage)
                        {
                            #region [Sincronizacion de inserciones servidor -> local]

                            // Fase A1 - Procesar las inserciones del servidor en local
                            // Entidades: Calendario, Departamento, Moneda, TipoGasto, y EstadoLiquidación
                            case 1:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Calendario:
                                        this.SyncCalendarioToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;

                                    case (int)TrackingEntry.EntidadType.Departamento:
                                        this.SyncDepartamentoToLocal(entry, iteration == 1);
                                        if (iteration > 1)
                                            discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;

                                    case (int)TrackingEntry.EntidadType.Moneda:
                                        this.SyncMonedaToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.TipoGasto:
                                        this.SyncTipoGastoToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.EstadoLiquidacion:
                                        this.SyncEstadoLiquidacionToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase A2 - Procesar las inserciones del servidor en local
                            // Entidades: Usuario, CambioMoneda
                            case 2:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Usuario:
                                        this.SyncUsuarioToLocal(entry, iteration == 1);
                                        if (iteration > 1)
                                            discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.CambioMoneda:
                                        this.SyncCambioMonedaToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase A3 - Procesar las inserciones del servidor en local
                            // Entidades: Liquidacion
                            case 3:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Liquidacion:
                                        this.SyncLiquidacionToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase A4 - Procesar las inserciones del servidor en local
                            // Entidades: Gasto, CambiosEstado
                            case 4:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Gasto:
                                        this.SyncGastoToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.CambioEstadoLiquidacion:
                                        this.SyncCambioEstadoLiquidacionToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase A5 - Procesar las inserciones del servidor en local
                            // Entidades: Notificacion
                            case 5:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Notificacion:
                                        this.SyncNotificacionToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            #endregion [Sincronizacion de inserciones servidor -> local]

                            #region [Sincronizacion de actualizaciones servidor -> local]

                            // Fase B1 - Procesar las actualizaciones del servidor en local
                            // Entidades: Calendario, Departamento, Moneda, TipoGasto, y EstadoLiquidación
                            case 6:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Calendario:
                                        this.SyncCalendarioToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.Departamento:
                                        this.SyncDepartamentoToLocal(entry, iteration == 1);
                                        if (iteration > 1)
                                            discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.Moneda:
                                        this.SyncMonedaToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.TipoGasto:
                                        this.SyncTipoGastoToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.EstadoLiquidacion:
                                        this.SyncEstadoLiquidacionToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase B2 - Procesar las actualizaciones del servidor en local
                            // Entidades: Usuario, CambioMoneda
                            case 7:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Usuario:
                                        this.SyncUsuarioToLocal(entry, iteration == 1);
                                        if (iteration > 1)
                                            discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.CambioMoneda:
                                        this.SyncCambioMonedaToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase B3 - Procesar las actualizaciones del servidor en local
                            // Entidades: Liquidacion
                            case 8:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Liquidacion:
                                        this.SyncLiquidacionToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase B4 - Procesar las actualizaciones del servidor en local
                            // Entidades: Gasto, CambiosEstado
                            case 9:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Gasto:
                                        this.SyncGastoToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.CambioEstadoLiquidacion:
                                        this.SyncCambioEstadoLiquidacionToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase B5 - Procesar las actualizaciones del servidor en local
                            // Entidades: Notificacion
                            case 10:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Notificacion:
                                        this.SyncNotificacionToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            #endregion [Sincronizacion de actualizaciones servidor -> local]

                            #region [Sincronizacion de eliminaciones servidor -> local]

                            // Fase C1 - Procesar las eliminaciones del servidor en local
                            // Entidades: Notificacion
                            case 11:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Notificacion:
                                        this.SyncNotificacionToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase C2 - Procesar las eliminaciones del servidor en local
                            // Entidades: Gasto, CambiosEstado
                            case 12:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Gasto:
                                        this.SyncGastoToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.CambioEstadoLiquidacion:
                                        this.SyncEstadoLiquidacionToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase C3 - Procesar las eliminaciones del servidor en local
                            // Entidades: Liquidacion
                            case 13:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Liquidacion:
                                        this.SyncLiquidacionToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase C4 - Procesar las eliminaciones del servidor en local
                            // Entidades: Usuario, CambioMoneda
                            case 14:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Usuario:
                                        this.SyncUsuarioToLocal(entry, false);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.CambioMoneda:
                                        this.SyncCambioMonedaToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase C5 - Procesar las eliminaciones del servidor en local
                            // Entidades: Calendario, Departamento, Moneda, TipoGasto, y EstadoLiquidación
                            case 15:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Calendario:
                                        this.SyncCalendarioToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.Departamento:
                                        this.SyncDepartamentoToLocal(entry, false);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.Moneda:
                                        this.SyncMonedaToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.TipoGasto:
                                        this.SyncTipoGastoToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.EstadoLiquidacion:
                                        this.SyncEstadoLiquidacionToLocal(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            #endregion [Sincronizacion de eliminaciones servidor -> local]

                            #region [Sincronizacion de inserciones local -> servidor]
                            // Fase A1 - Procesar las inserciones locales en el servidor
                            // Entidades: Calendario, Departamento, Moneda, TipoGasto, y EstadoLiquidación
                            case 16:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Calendario:
                                        this.SyncCalendarioToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;

                                    case (int)TrackingEntry.EntidadType.Departamento:
                                        this.SyncDepartamentoToServer(entry, iteration == 1);
                                        if (iteration > 1)
                                            discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;

                                    case (int)TrackingEntry.EntidadType.Moneda:
                                        this.SyncMonedaToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.TipoGasto:
                                        this.SyncTipoGastoToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.EstadoLiquidacion:
                                        this.SyncEstadoLiquidacionToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase A2 - Procesar las inserciones locales en el servidor
                            // Entidades: Usuario, CambioMoneda
                            case 17:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Usuario:
                                        this.SyncUsuarioToServer(entry, iteration == 1);
                                        if (iteration > 1)
                                            discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.CambioMoneda:
                                        this.SyncCambioMonedaToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase A3 - Procesar las inserciones locales en el servidor
                            // Entidades: Liquidacion
                            case 18:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Liquidacion:
                                        this.SyncLiquidacionToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase A4 - Procesar las inserciones locales en el servidor
                            // Entidades: Gasto, CambiosEstado
                            case 19:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Gasto:
                                        this.SyncGastoToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.CambioEstadoLiquidacion:
                                        this.SyncCambioEstadoLiquidacionToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase A5 - Procesar las inserciones locales en el servidor
                            // Entidades: Notificacion
                            case 20:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Notificacion:
                                        this.SyncNotificacionToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            #endregion [Sincronizacion de inserciones local -> servidor]

                            #region [Sincronizacion de actualizaciones local -> servidor]

                            // Fase B1 - Procesar las actualizaciones locales en el servidor
                            // Entidades: Calendario, Departamento, Moneda, TipoGasto, y EstadoLiquidación
                            case 21:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Calendario:
                                        this.SyncCalendarioToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.Departamento:
                                        this.SyncDepartamentoToServer(entry, iteration == 1);
                                        if (iteration > 1)
                                            discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.Moneda:
                                        this.SyncMonedaToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.TipoGasto:
                                        this.SyncTipoGastoToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.EstadoLiquidacion:
                                        this.SyncEstadoLiquidacionToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase B2 - Procesar las actualizaciones locales en el servidor
                            // Entidades: Usuario, CambioMoneda
                            case 22:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Usuario:
                                        this.SyncUsuarioToServer(entry, iteration == 1);
                                        if (iteration > 1)
                                            discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.CambioMoneda:
                                        this.SyncCambioMonedaToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase B3 - Procesar las actualizaciones locales en el servidor
                            // Entidades: Liquidacion
                            case 23:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Liquidacion:
                                        this.SyncLiquidacionToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase B4 - Procesar las actualizaciones locales en el servidor
                            // Entidades: Gasto, CambiosEstado
                            case 24:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Gasto:
                                        this.SyncGastoToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.CambioEstadoLiquidacion:
                                        this.SyncCambioEstadoLiquidacionToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase B5 - Procesar las actualizaciones locales en el servidor
                            // Entidades: Notificacion
                            case 25:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Notificacion:
                                        this.SyncNotificacionToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            #endregion [Sincronizacion de actualizaciones local -> servidor]

                            #region [Sincronizacion de eliminaciones local -> servidor]

                            // Fase C1 - Procesar las eliminaciones locales en el servidor
                            // Entidades: Notificacion
                            case 26:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Notificacion:
                                        this.SyncNotificacionToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase C2 - Procesar las eliminaciones locales en el servidor
                            // Entidades: Gasto, CambiosEstado
                            case 27:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Gasto:
                                        this.SyncGastoToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.CambioEstadoLiquidacion:
                                        this.SyncEstadoLiquidacionToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase C3 - Procesar las eliminaciones locales en el servidor
                            // Entidades: Liquidacion
                            case 28:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Liquidacion:
                                        this.SyncLiquidacionToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase C4 - Procesar las eliminaciones locales en el servidor
                            // Entidades: Usuario, CambioMoneda
                            case 29:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Usuario:
                                        this.SyncUsuarioToServer(entry, false);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.CambioMoneda:
                                        this.SyncCambioMonedaToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            // Fase C5 - Procesar las eliminaciones locales en el servidor
                            // Entidades: Calendario, Departamento, Moneda, TipoGasto, y EstadoLiquidación
                            case 30:
                                switch (entry.Entidad)
                                {
                                    case (int)TrackingEntry.EntidadType.Calendario:
                                        this.SyncCalendarioToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.Departamento:
                                        this.SyncDepartamentoToServer(entry, false);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.Moneda:
                                        this.SyncMonedaToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.TipoGasto:
                                        this.SyncTipoGastoToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                    case (int)TrackingEntry.EntidadType.EstadoLiquidacion:
                                        this.SyncEstadoLiquidacionToServer(entry);
                                        discardedTrackingEntryDic.Add(entry.IdTracking, entry);
                                        break;
                                }
                                break;

                            #endregion [Sincronizacion de eliminaciones local -> servidor]

                        }

                        this.Commit();

                    }

                }
                #endregion [Sincronización por fases]
            }
            Debug.WriteLine("Cliente: Sincronización completa finalizada");

            return entitySynchronizedCount;
        }

        private void SyncCalendarioToLocal(TrackingEntry entryRemote)
        {
            // Obtener los objetos de entidad.
            Calendario calendarioRemote = this.EntitySyncRepositoryRemote.GetCalendario((Guid)entryRemote.Guid);
            Calendario calendarioLocal = this.CalendarioRepositoryLocal.GetCalendario((Guid)entryRemote.Guid);

            // Obtener el objeto de seguimiento local
            TrackingEntry entryLocal = this.TrackingRepositoryLocal.GetTrackingEntry(entryRemote.IdTracking);

            // Eliminar las entidades relacionadas.
            calendarioRemote.Liquidaciones = null;
            //calendarioRemote.Usuarios = null;

            switch (entryRemote.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en local. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (calendarioLocal != null)
                    {
                        this.CalendarioRepositoryLocal.Detach(calendarioLocal);
                        this.CalendarioRepositoryLocal.UpdateCalendario(calendarioRemote);
                    }
                    else
                        this.CalendarioRepositoryLocal.InsertCalendario(calendarioRemote);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (calendarioLocal != null)
                        this.CalendarioRepositoryLocal.DeleteCalendario(calendarioRemote);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la entrada del tracking.                    
            if (entryLocal != null)
            {
                this.TrackingRepositoryLocal.Detach(entryLocal);
                this.TrackingRepositoryLocal.UpdateTrackingEntry(entryRemote);
            }
            else
                this.TrackingRepositoryLocal.InsertTrackingEntry(entryRemote);
        }

        private void SyncCalendarioToServer(TrackingEntry entryLocal)
        {
            // Obtener los objetos de entidad.
            Calendario calendarioRemote = this.EntitySyncRepositoryRemote.GetCalendario((Guid)entryLocal.Guid);
            Calendario calendarioLocal = this.CalendarioRepositoryLocal.GetCalendario((Guid)entryLocal.Guid);

            // Obtener el objeto de seguimiento Remoto
            TrackingEntry entryRemote = this.TrackingRepositoryRemote.GetTrackingEntry(entryLocal.IdTracking);

            // Eliminar las entidades relacionadas.
            calendarioLocal.Liquidaciones = null;
            //calendarioLocal.Usuarios = null;

            switch (entryLocal.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en remoto. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (calendarioRemote != null)
                        this.EntitySyncRepositoryRemote.UpdateCalendario(calendarioLocal);
                    else
                        this.EntitySyncRepositoryRemote.InsertCalendario(calendarioLocal);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (calendarioRemote != null)
                        this.EntitySyncRepositoryRemote.DeleteCalendario(calendarioLocal);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la inserción y actualización de la entrada del tracking.
            if (entryRemote != null)
                this.TrackingRepositoryRemote.UpdateTrackingEntry(entryLocal);
            else
                this.TrackingRepositoryRemote.InsertTrackingEntry(entryLocal);
        }

        private void SyncDepartamentoToLocal(TrackingEntry entryRemote, bool isFirstIteration)
        {
            // Obtener los objetos de entidad.
            Departamento departamentoRemote = this.EntitySyncRepositoryRemote.GetDepartamento((Guid)entryRemote.Guid);
            Departamento departamentoLocal = this.DepartamentoRepositoryLocal.GetDepartamento((Guid)entryRemote.Guid);

            // Obtener el objeto de seguimiento local
            TrackingEntry entryLocal = this.TrackingRepositoryLocal.GetTrackingEntry(entryRemote.IdTracking);

            // Si la entidad remota es null, es porque se eliminó posteriormente. Si no es así tratarla.
            if (departamentoLocal != null)
            {
                // Eliminar las entidades relacionadas.
                departamentoRemote.Responsable = null;
                departamentoRemote.Superdepartamento = null;
                departamentoRemote.Subdepartamentos = null;
                departamentoRemote.Usuarios = null;

                // La actualización en dos fases, inserta en la primera fase la entidad sin relaciones, y en la segunda
                // fase actualiza las referencias a las otras entidades que ya exiten con seguridad.
                if (isFirstIteration)
                {
                    departamentoRemote.IdResponsable = null;
                    departamentoRemote.IdSuperdepartamento = null;
                }

                switch (entryRemote.Operacion)
                {
                    // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                    case (int)TrackingEntry.OperacionType.Insercion:
                    case (int)TrackingEntry.OperacionType.Actualizacion:
                        // Comprobar si la entidad existe en local. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                        if (departamentoLocal != null)
                        {
                            this.DepartamentoRepositoryLocal.Detach(departamentoLocal);
                            this.DepartamentoRepositoryLocal.UpdateDepartamento(departamentoRemote);
                        }
                        else
                            this.DepartamentoRepositoryLocal.InsertDepartamento(departamentoRemote);
                        break;

                    case (int)TrackingEntry.OperacionType.Borrado:
                        if (departamentoLocal != null)
                            this.DepartamentoRepositoryLocal.DeleteDepartamento(departamentoRemote);
                        break;

                    default:
                        throw new InvalidOperationException("Operación de sincroniación no esperada.");
                }
            }

            if (isFirstIteration)
                return;

            // Idem con la entrada del tracking.                    
            if (entryLocal != null)
            {
                this.TrackingRepositoryLocal.Detach(entryLocal);
                this.TrackingRepositoryLocal.UpdateTrackingEntry(entryRemote);
            }
            else
                this.TrackingRepositoryLocal.InsertTrackingEntry(entryRemote);
        }

        private void SyncDepartamentoToServer(TrackingEntry entryLocal, bool isFirstIteration)
        {
            // Obtener los objetos de entidad.
            Departamento departamentoRemote = this.EntitySyncRepositoryRemote.GetDepartamento((Guid)entryLocal.Guid);
            Departamento departamentoLocal = this.DepartamentoRepositoryLocal.GetDepartamento((Guid)entryLocal.Guid);            

            // Obtener el objeto de seguimiento Remoto
            TrackingEntry entryRemote = this.TrackingRepositoryRemote.GetTrackingEntry(entryLocal.IdTracking);

            // Si la entidad local es null, es porque se eliminó posteriormente. Si no es así tratarla.
            if (departamentoLocal != null)
            {
                // Eliminar las entidades relacionadas.
                departamentoLocal.Responsable = null;
                departamentoLocal.Superdepartamento = null;
                departamentoLocal.Subdepartamentos = null;
                departamentoLocal.Usuarios = null;

                // La actualización en dos fases, inserta en la primera fase la entidad sin relaciones, y en la segunda
                // fase actualiza las referencias a las otras entidades que ya exiten con seguridad.
                if (isFirstIteration)
                {
                    departamentoLocal.IdResponsable = null;
                    departamentoLocal.IdSuperdepartamento = null;
                }

                switch (entryLocal.Operacion)
                {
                    // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                    case (int)TrackingEntry.OperacionType.Insercion:
                    case (int)TrackingEntry.OperacionType.Actualizacion:
                        // Comprobar si la entidad existe en remoto. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                        if (departamentoRemote != null)
                            this.EntitySyncRepositoryRemote.UpdateDepartamento(departamentoLocal);
                        else
                            this.EntitySyncRepositoryRemote.InsertDepartamento(departamentoLocal);
                        break;

                    case (int)TrackingEntry.OperacionType.Borrado:
                        if (departamentoRemote != null)
                            this.EntitySyncRepositoryRemote.DeleteDepartamento(departamentoLocal);
                        break;

                    default:
                        throw new InvalidOperationException("Operación de sincroniación no esperada.");
                }

                // Resturar el objeto local al estadio inicial, es decir recuperar las relaciones.
                this.DepartamentoRepositoryLocal.RefreshDepartamento(departamentoLocal);
            }            

            if (isFirstIteration)
                return;

            // Idem con la inserción y actualización de la entrada del tracking.
            if (entryRemote != null)
                this.TrackingRepositoryRemote.UpdateTrackingEntry(entryLocal);
            else
                this.TrackingRepositoryRemote.InsertTrackingEntry(entryLocal);
        }

        private void SyncMonedaToLocal(TrackingEntry entryRemote)
        {
            // Obtener los objetos de entidad.
            Moneda monedaRemote = this.EntitySyncRepositoryRemote.GetMoneda((int)entryRemote.Id);
            Moneda monedaLocal = this.MonedaRepositoryLocal.GetMoneda((int)entryRemote.Id);

            // Obtener el objeto de seguimiento local
            TrackingEntry entryLocal = this.TrackingRepositoryLocal.GetTrackingEntry(entryRemote.IdTracking);

            // Eliminar las entidades relacionadas.
            monedaRemote.CambiosMoneda = null;

            switch (entryRemote.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en local. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (monedaLocal != null)
                    {
                        this.MonedaRepositoryLocal.Detach(monedaLocal);
                        this.MonedaRepositoryLocal.UpdateMoneda(monedaRemote);
                    }
                    else
                        this.MonedaRepositoryLocal.InsertMoneda(monedaRemote);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (monedaLocal != null)
                        this.MonedaRepositoryLocal.DeleteMoneda(monedaRemote);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la entrada del tracking.                    
            if (entryLocal != null)
            {
                this.TrackingRepositoryLocal.Detach(entryLocal);
                this.TrackingRepositoryLocal.UpdateTrackingEntry(entryRemote);
            }
            else
                this.TrackingRepositoryLocal.InsertTrackingEntry(entryRemote);
        }

        private void SyncMonedaToServer(TrackingEntry entryLocal)
        {
            // Obtener los objetos de entidad.
            Moneda monedaRemote = this.EntitySyncRepositoryRemote.GetMoneda((int)entryLocal.Id);
            Moneda monedaLocal = this.MonedaRepositoryLocal.GetMoneda((int)entryLocal.Id);

            // Obtener el objeto de seguimiento Remoto
            TrackingEntry entryRemote = this.TrackingRepositoryRemote.GetTrackingEntry(entryLocal.IdTracking);

            // Eliminar las entidades relacionadas.
            monedaLocal.CambiosMoneda = null;

            switch (entryLocal.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en remoto. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (monedaRemote != null)
                        this.EntitySyncRepositoryRemote.UpdateMoneda(monedaLocal);
                    else
                        this.EntitySyncRepositoryRemote.InsertMoneda(monedaLocal);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (monedaRemote != null)
                        this.EntitySyncRepositoryRemote.DeleteMoneda(monedaLocal);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la inserción y actualización de la entrada del tracking.
            if (entryRemote != null)
                this.TrackingRepositoryRemote.UpdateTrackingEntry(entryLocal);
            else
                this.TrackingRepositoryRemote.InsertTrackingEntry(entryLocal);
        }
        
        private void SyncTipoGastoToLocal(TrackingEntry entryRemote)
        {
            // Obtener los objetos de entidad.
            TipoGasto tipoGastoRemote = this.EntitySyncRepositoryRemote.GetTipoGasto((int)entryRemote.Id);
            TipoGasto tipoGastoLocal = this.TipoGastoRepositoryLocal.GetTipoGasto((int)entryRemote.Id);

            // Obtener el objeto de seguimiento local
            TrackingEntry entryLocal = this.TrackingRepositoryLocal.GetTrackingEntry(entryRemote.IdTracking);

            // Eliminar las entidades relacionadas.
            tipoGastoRemote.Gastos = null;

            switch (entryRemote.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en local. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (tipoGastoLocal != null)
                    {
                        this.TipoGastoRepositoryLocal.Detach(tipoGastoLocal);
                        this.TipoGastoRepositoryLocal.UpdateTipoGasto(tipoGastoRemote);
                    }
                    else
                        this.TipoGastoRepositoryLocal.InsertTipoGasto(tipoGastoRemote);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (tipoGastoLocal != null)
                        this.TipoGastoRepositoryLocal.DeleteTipoGasto(tipoGastoRemote);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la entrada del tracking.                    
            if (entryLocal != null)
            {
                this.TrackingRepositoryLocal.Detach(entryLocal);
                this.TrackingRepositoryLocal.UpdateTrackingEntry(entryRemote);
            }
            else
                this.TrackingRepositoryLocal.InsertTrackingEntry(entryRemote);
        }

        private void SyncTipoGastoToServer(TrackingEntry entryLocal)
        {
            // Obtener los objetos de entidad.
            TipoGasto tipoGastoRemote = this.EntitySyncRepositoryRemote.GetTipoGasto((int)entryLocal.Id);
            TipoGasto tipoGastoLocal = this.TipoGastoRepositoryLocal.GetTipoGasto((int)entryLocal.Id);

            // Obtener el objeto de seguimiento Remoto
            TrackingEntry entryRemote = this.TrackingRepositoryRemote.GetTrackingEntry(entryLocal.IdTracking);

            // Eliminar las entidades relacionadas.
            tipoGastoLocal.Gastos = null;

            switch (entryLocal.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en remoto. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (tipoGastoRemote != null)
                        this.EntitySyncRepositoryRemote.UpdateTipoGasto(tipoGastoLocal);
                    else
                        this.EntitySyncRepositoryRemote.InsertTipoGasto(tipoGastoLocal);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (tipoGastoRemote != null)
                        this.EntitySyncRepositoryRemote.DeleteTipoGasto(tipoGastoLocal);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la inserción y actualización de la entrada del tracking.
            if (entryRemote != null)
                this.TrackingRepositoryRemote.UpdateTrackingEntry(entryLocal);
            else
                this.TrackingRepositoryRemote.InsertTrackingEntry(entryLocal);
        }

        private void SyncEstadoLiquidacionToLocal(TrackingEntry entryRemote)
        {
            // Obtener los objetos de entidad.
            EstadoLiquidacion estadoLiquidacionRemote = this.EntitySyncRepositoryRemote.GetEstadoLiquidacion((int)entryRemote.Id);
            EstadoLiquidacion estadoLiquidacionLocal = this.EstadoLiquidacionRepositoryLocal.GetEstadoLiquidacion((int)entryRemote.Id);

            // Obtener el objeto de seguimiento local
            TrackingEntry entryLocal = this.TrackingRepositoryLocal.GetTrackingEntry(entryRemote.IdTracking);

            // Eliminar las entidades relacionadas.
            estadoLiquidacionRemote.CambiosEstadoAnteriorLiquidacion = null;
            estadoLiquidacionRemote.CambiosEstadoNuevoLiquidacion = null;
            estadoLiquidacionRemote.Liquidaciones = null;

            switch (entryRemote.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en local. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (estadoLiquidacionLocal != null)
                    {
                        this.EstadoLiquidacionRepositoryLocal.Detach(estadoLiquidacionLocal);
                        this.EstadoLiquidacionRepositoryLocal.UpdateEstadoLiquidacion(estadoLiquidacionRemote);
                    }
                    else
                        this.EstadoLiquidacionRepositoryLocal.InsertEstadoLiquidacion(estadoLiquidacionRemote);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (estadoLiquidacionLocal != null)
                        this.EstadoLiquidacionRepositoryLocal.DeleteEstadoLiquidacion(estadoLiquidacionRemote);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la entrada del tracking.                    
            if (entryLocal != null)
            {
                this.TrackingRepositoryLocal.Detach(entryLocal);
                this.TrackingRepositoryLocal.UpdateTrackingEntry(entryRemote);
            }
            else
                this.TrackingRepositoryLocal.InsertTrackingEntry(entryRemote);
        }

        private void SyncEstadoLiquidacionToServer(TrackingEntry entryLocal)
        {
            // Obtener los objetos de entidad.
            EstadoLiquidacion estadoLiquidacionRemote = this.EntitySyncRepositoryRemote.GetEstadoLiquidacion((int)entryLocal.Id);
            EstadoLiquidacion estadoLiquidacionLocal = this.EstadoLiquidacionRepositoryLocal.GetEstadoLiquidacion((int)entryLocal.Id);

            // Obtener el objeto de seguimiento Remoto
            TrackingEntry entryRemote = this.TrackingRepositoryRemote.GetTrackingEntry(entryLocal.IdTracking);

            // Eliminar las entidades relacionadas.
            estadoLiquidacionLocal.CambiosEstadoAnteriorLiquidacion = null;
            estadoLiquidacionLocal.CambiosEstadoNuevoLiquidacion = null;
            estadoLiquidacionLocal.Liquidaciones = null;

            switch (entryLocal.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en remoto. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (estadoLiquidacionRemote != null)
                        this.EntitySyncRepositoryRemote.UpdateEstadoLiquidacion(estadoLiquidacionLocal);
                    else
                        this.EntitySyncRepositoryRemote.InsertEstadoLiquidacion(estadoLiquidacionLocal);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (estadoLiquidacionRemote != null)
                        this.EntitySyncRepositoryRemote.DeleteEstadoLiquidacion(estadoLiquidacionLocal);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la inserción y actualización de la entrada del tracking.
            if (entryRemote != null)
                this.TrackingRepositoryRemote.UpdateTrackingEntry(entryLocal);
            else
                this.TrackingRepositoryRemote.InsertTrackingEntry(entryLocal);
        }

        private void SyncUsuarioToLocal(TrackingEntry entryRemote, bool isFirstIteration)
        {
            // Obtener los objetos de entidad.
            Usuario usuarioRemote = this.EntitySyncRepositoryRemote.GetUsuario((Guid)entryRemote.Guid);
            Usuario usuarioLocal = this.UsuarioRepositoryLocal.GetUsuario((Guid)entryRemote.Guid);

            // Obtener el objeto de seguimiento local
            TrackingEntry entryLocal = this.TrackingRepositoryLocal.GetTrackingEntry(entryRemote.IdTracking);

            // Si la entidad remota es null, es porque se eliminó posteriormente. Si no es así tratarla.
            if (usuarioRemote != null)
            {
                // Eliminar las entidades relacionadas.
                //usuarioRemote.Calendario = null;
                usuarioRemote.Departamento = null;
                usuarioRemote.DepartamentosSupervisados = null;
                usuarioRemote.Liquidaciones = null;

                // La actualización en dos fases, inserta en la primera fase la entidad sin relaciones, y en la segunda
                // fase actualiza las referencias a las otras entidades que ya exiten con seguridad.
                if (isFirstIteration)
                {
                    //usuarioRemote.IdCalendario = null;
                    usuarioRemote.IdDepartamento = null;
                }

                switch (entryRemote.Operacion)
                {
                    // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                    case (int)TrackingEntry.OperacionType.Insercion:
                    case (int)TrackingEntry.OperacionType.Actualizacion:
                        // Comprobar si la entidad existe en local. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                        if (usuarioLocal != null)
                        {
                            this.UsuarioRepositoryLocal.Detach(usuarioLocal);
                            this.UsuarioRepositoryLocal.UpdateUsuario(usuarioRemote);
                        }
                        else
                            this.UsuarioRepositoryLocal.InsertUsuario(usuarioRemote);
                        break;

                    case (int)TrackingEntry.OperacionType.Borrado:
                        if (usuarioLocal != null)
                            this.UsuarioRepositoryLocal.DeleteUsuario(usuarioRemote);
                        break;

                    default:
                        throw new InvalidOperationException("Operación de sincroniación no esperada.");
                }
            }

            if (isFirstIteration)
                return;

            // Idem con la entrada del tracking.                    
            if (entryLocal != null)
            {
                this.TrackingRepositoryLocal.Detach(entryLocal);
                this.TrackingRepositoryLocal.UpdateTrackingEntry(entryRemote);
            }
            else
                this.TrackingRepositoryLocal.InsertTrackingEntry(entryRemote);
        }

        private void SyncUsuarioToServer(TrackingEntry entryLocal, bool isFirstIteration)
        {
            // Obtener los objetos de entidad.
            Usuario usuarioRemote = this.EntitySyncRepositoryRemote.GetUsuario((Guid)entryLocal.Guid);
            Usuario usuarioLocal = this.UsuarioRepositoryLocal.GetUsuario((Guid)entryLocal.Guid);

            // Obtener el objeto de seguimiento Remoto
            TrackingEntry entryRemote = this.TrackingRepositoryRemote.GetTrackingEntry(entryLocal.IdTracking);

            // Si la entidad local es null, es porque se eliminó posteriormente. Si no es así tratarla.
            if (usuarioLocal != null)
            {
                // Eliminar las entidades relacionadas.
                //usuarioLocal.Calendario = null;
                usuarioLocal.Departamento = null;
                usuarioLocal.DepartamentosSupervisados = null;
                usuarioLocal.Liquidaciones = null;

                // La actualización en dos fases, inserta en la primera fase la entidad sin relaciones, y en la segunda
                // fase actualiza las referencias a las otras entidades que ya exiten con seguridad.
                if (isFirstIteration)
                {
                    //usuarioLocal.IdCalendario = null;
                    usuarioLocal.IdDepartamento = null;
                }

                switch (entryLocal.Operacion)
                {
                    // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                    case (int)TrackingEntry.OperacionType.Insercion:
                    case (int)TrackingEntry.OperacionType.Actualizacion:
                        // Comprobar si la entidad existe en remoto. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                        if (usuarioRemote != null)
                            this.EntitySyncRepositoryRemote.UpdateUsuario(usuarioLocal);
                        else
                            this.EntitySyncRepositoryRemote.InsertUsuario(usuarioLocal);
                        break;

                    case (int)TrackingEntry.OperacionType.Borrado:
                        if (usuarioRemote != null)
                            this.EntitySyncRepositoryRemote.DeleteUsuario(usuarioLocal);
                        break;

                    default:
                        throw new InvalidOperationException("Operación de sincroniación no esperada.");
                }

                // Resturar el objeto local al estadio inicial, es decir recuperar las relaciones.
                this.UsuarioRepositoryLocal.RefreshUsuario(usuarioLocal);
            }

            if (isFirstIteration)
                return;

            // Idem con la inserción y actualización de la entrada del tracking.
            if (entryRemote != null)
                this.TrackingRepositoryRemote.UpdateTrackingEntry(entryLocal);
            else
                this.TrackingRepositoryRemote.InsertTrackingEntry(entryLocal);
        }

        private void SyncCambioMonedaToLocal(TrackingEntry entryRemote)
        {
            // Obtener los objetos de entidad.
            CambioMoneda cambioMonedaRemote = this.EntitySyncRepositoryRemote.GetCambioMoneda((Guid)entryRemote.Guid);
            CambioMoneda cambioMonedaLocal = this.CambioMonedaRepositoryLocal.GetCambioMoneda((Guid)entryRemote.Guid);

            // Obtener el objeto de seguimiento local
            TrackingEntry entryLocal = this.TrackingRepositoryLocal.GetTrackingEntry(entryRemote.IdTracking);

            // Eliminar las entidades relacionadas.
            cambioMonedaRemote.Gastos = null;
            cambioMonedaRemote.Moneda = null;

            switch (entryRemote.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en local. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (cambioMonedaLocal != null)
                    {
                        this.CambioMonedaRepositoryLocal.Detach(cambioMonedaLocal);
                        this.CambioMonedaRepositoryLocal.UpdateCambioMoneda(cambioMonedaRemote);
                    }
                    else
                        this.CambioMonedaRepositoryLocal.InsertCambioMoneda(cambioMonedaRemote);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (cambioMonedaLocal != null)
                        this.CambioMonedaRepositoryLocal.DeleteCambioMoneda(cambioMonedaRemote);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la entrada del tracking.                    
            if (entryLocal != null)
            {
                this.TrackingRepositoryLocal.Detach(entryLocal);
                this.TrackingRepositoryLocal.UpdateTrackingEntry(entryRemote);
            }
            else
                this.TrackingRepositoryLocal.InsertTrackingEntry(entryRemote);
        }

        private void SyncCambioMonedaToServer(TrackingEntry entryLocal)
        {
            // Obtener los objetos de entidad.
            CambioMoneda cambioMonedaRemote = this.EntitySyncRepositoryRemote.GetCambioMoneda((Guid)entryLocal.Guid);
            CambioMoneda cambioMonedaLocal = this.CambioMonedaRepositoryLocal.GetCambioMoneda((Guid)entryLocal.Guid);

            // Obtener el objeto de seguimiento Remoto
            TrackingEntry entryRemote = this.TrackingRepositoryRemote.GetTrackingEntry(entryLocal.IdTracking);

            // Eliminar las entidades relacionadas.
            cambioMonedaLocal.Gastos = null;
            cambioMonedaLocal.Moneda = null;

            switch (entryLocal.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en remoto. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (cambioMonedaRemote != null)
                        this.EntitySyncRepositoryRemote.UpdateCambioMoneda(cambioMonedaLocal);
                    else
                        this.EntitySyncRepositoryRemote.InsertCambioMoneda(cambioMonedaLocal);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (cambioMonedaRemote != null)
                        this.EntitySyncRepositoryRemote.DeleteCambioMoneda(cambioMonedaLocal);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la inserción y actualización de la entrada del tracking.
            if (entryRemote != null)
                this.TrackingRepositoryRemote.UpdateTrackingEntry(entryLocal);
            else
                this.TrackingRepositoryRemote.InsertTrackingEntry(entryLocal);
        }

        private void SyncLiquidacionToLocal(TrackingEntry entryRemote)
        {
            // Obtener los objetos de entidad.
            Liquidacion liquidacionRemote = this.EntitySyncRepositoryRemote.GetLiquidacion((Guid)entryRemote.Guid);
            Liquidacion liquidacionLocal = this.LiquidacionRepositoryLocal.GetLiquidacion((Guid)entryRemote.Guid);

            // Obtener el objeto de seguimiento local
            TrackingEntry entryLocal = this.TrackingRepositoryLocal.GetTrackingEntry(entryRemote.IdTracking);

            // Eliminar las entidades relacionadas.
            liquidacionRemote.Calendario = null;
            liquidacionRemote.CambiosEstado = null;
            liquidacionRemote.Gastos = null;
            liquidacionRemote.Usuario = null;

            switch (entryRemote.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en local. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (liquidacionLocal != null)
                    {
                        this.LiquidacionRepositoryLocal.Detach(liquidacionLocal);
                        this.LiquidacionRepositoryLocal.UpdateLiquidacion(liquidacionRemote);
                    }
                    else
                        this.LiquidacionRepositoryLocal.InsertLiquidacion(liquidacionRemote);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (liquidacionLocal != null)
                        this.LiquidacionRepositoryLocal.DeleteLiquidacion(liquidacionRemote);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la entrada del tracking.                    
            if (entryLocal != null)
            {
                this.TrackingRepositoryLocal.Detach(entryLocal);
                this.TrackingRepositoryLocal.UpdateTrackingEntry(entryRemote);
            }
            else
                this.TrackingRepositoryLocal.InsertTrackingEntry(entryRemote);
        }

        private void SyncLiquidacionToServer(TrackingEntry entryLocal)
        {
            // Obtener los objetos de entidad.
            Liquidacion liquidacionRemote = this.EntitySyncRepositoryRemote.GetLiquidacion((Guid)entryLocal.Guid);
            Liquidacion liquidacionLocal = this.LiquidacionRepositoryLocal.GetLiquidacion((Guid)entryLocal.Guid);

            // Obtener el objeto de seguimiento Remoto
            TrackingEntry entryRemote = this.TrackingRepositoryRemote.GetTrackingEntry(entryLocal.IdTracking);

            // Eliminar las entidades relacionadas.
            liquidacionLocal.Calendario = null;
            liquidacionLocal.CambiosEstado = null;
            liquidacionLocal.Gastos = null;
            liquidacionLocal.Usuario = null;

            switch (entryLocal.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en remoto. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (liquidacionRemote != null)
                        this.EntitySyncRepositoryRemote.UpdateLiquidacion(liquidacionLocal);
                    else
                        this.EntitySyncRepositoryRemote.InsertLiquidacion(liquidacionLocal);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (liquidacionRemote != null)
                        this.EntitySyncRepositoryRemote.DeleteLiquidacion(liquidacionLocal);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la inserción y actualización de la entrada del tracking.
            if (entryRemote != null)
                this.TrackingRepositoryRemote.UpdateTrackingEntry(entryLocal);
            else
                this.TrackingRepositoryRemote.InsertTrackingEntry(entryLocal);
        }

        private void SyncGastoToLocal(TrackingEntry entryRemote)
        {
            // Obtener los objetos de entidad.
            Gasto gastoRemote = this.EntitySyncRepositoryRemote.GetGasto((Guid)entryRemote.Guid);
            Gasto gastoLocal = this.GastoRepositoryLocal.GetGasto((Guid)entryRemote.Guid);

            // Eliminar las entidades relacionadas.
            gastoRemote.CambioMoneda = null;
            gastoRemote.Liquidacion = null;
            gastoRemote.Tipo = null;

            // Obtener el objeto de seguimiento local
            TrackingEntry entryLocal = this.TrackingRepositoryLocal.GetTrackingEntry(entryRemote.IdTracking);

            switch (entryRemote.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en local. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (gastoLocal != null)
                    {
                        this.GastoRepositoryLocal.Detach(gastoLocal);
                        this.GastoRepositoryLocal.UpdateGasto(gastoRemote);
                    }
                    else
                        this.GastoRepositoryLocal.InsertGasto(gastoRemote);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (gastoLocal != null)
                        this.GastoRepositoryLocal.DeleteGasto(gastoRemote);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la entrada del tracking.                    
            if (entryLocal != null)
            {
                this.TrackingRepositoryLocal.Detach(entryLocal);
                this.TrackingRepositoryLocal.UpdateTrackingEntry(entryRemote);
            }
            else
                this.TrackingRepositoryLocal.InsertTrackingEntry(entryRemote);
        }

        private void SyncGastoToServer(TrackingEntry entryLocal)
        {
            // Obtener los objetos de entidad.
            Gasto gastoRemote = this.EntitySyncRepositoryRemote.GetGasto((Guid)entryLocal.Guid);
            Gasto gastoLocal = this.GastoRepositoryLocal.GetGasto((Guid)entryLocal.Guid);

            // Obtener el objeto de seguimiento Remoto
            TrackingEntry entryRemote = this.TrackingRepositoryRemote.GetTrackingEntry(entryLocal.IdTracking);

            // Eliminar las entidades relacionadas.
            gastoLocal.CambioMoneda = null;
            gastoLocal.Liquidacion = null;
            gastoLocal.Tipo = null;

            switch (entryLocal.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en remoto. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (gastoRemote != null)
                        this.EntitySyncRepositoryRemote.UpdateGasto(gastoLocal);
                    else
                        this.EntitySyncRepositoryRemote.InsertGasto(gastoLocal);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (gastoRemote != null)
                        this.EntitySyncRepositoryRemote.DeleteGasto(gastoLocal);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la inserción y actualización de la entrada del tracking.
            if (entryRemote != null)
                this.TrackingRepositoryRemote.UpdateTrackingEntry(entryLocal);
            else
                this.TrackingRepositoryRemote.InsertTrackingEntry(entryLocal);
        }

        private void SyncCambioEstadoLiquidacionToLocal(TrackingEntry entryRemote)
        {
            // Obtener los objetos de entidad.
            CambioEstadoLiquidacion cambioEstadoLiquidacionRemote = this.EntitySyncRepositoryRemote.GetCambioEstadoLiquidacion((Guid)entryRemote.Guid);
            CambioEstadoLiquidacion cambioEstadoLiquidacionLocal = this.CambioEstadoLiquidacionRepositoryLocal.GetCambioEstadoLiquidacion((Guid)entryRemote.Guid);

            // Obtener el objeto de seguimiento local
            TrackingEntry entryLocal = this.TrackingRepositoryLocal.GetTrackingEntry(entryRemote.IdTracking);

            // Eliminar las entidades relacionadas.
            cambioEstadoLiquidacionRemote.EstadoAnteriorLiquidacion = null;
            cambioEstadoLiquidacionRemote.EstadoNuevoLiquidacion = null;
            cambioEstadoLiquidacionRemote.Liquidacion = null;
            cambioEstadoLiquidacionRemote.Notificaciones = null;

            switch (entryRemote.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en local. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (cambioEstadoLiquidacionLocal != null)
                    {
                        this.CambioEstadoLiquidacionRepositoryLocal.Detach(cambioEstadoLiquidacionLocal);
                        this.CambioEstadoLiquidacionRepositoryLocal.UpdateCambioEstadoLiquidacion(cambioEstadoLiquidacionRemote);
                    }
                    else
                        this.CambioEstadoLiquidacionRepositoryLocal.InsertCambioEstadoLiquidacion(cambioEstadoLiquidacionRemote);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (cambioEstadoLiquidacionLocal != null)
                        this.CambioEstadoLiquidacionRepositoryLocal.DeleteCambioEstadoLiquidacion(cambioEstadoLiquidacionRemote);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la entrada del tracking.                    
            if (entryLocal != null)
            {
                this.TrackingRepositoryLocal.Detach(entryLocal);
                this.TrackingRepositoryLocal.UpdateTrackingEntry(entryRemote);
            }
            else
                this.TrackingRepositoryLocal.InsertTrackingEntry(entryRemote);
        }

        private void SyncCambioEstadoLiquidacionToServer(TrackingEntry entryLocal)
        {
            // Obtener los objetos de entidad.
            CambioEstadoLiquidacion cambioEstadoLiquidacionRemote = this.EntitySyncRepositoryRemote.GetCambioEstadoLiquidacion((Guid)entryLocal.Guid);
            CambioEstadoLiquidacion cambioEstadoLiquidacionLocal = this.CambioEstadoLiquidacionRepositoryLocal.GetCambioEstadoLiquidacion((Guid)entryLocal.Guid);

            // Obtener el objeto de seguimiento Remoto
            TrackingEntry entryRemote = this.TrackingRepositoryRemote.GetTrackingEntry(entryLocal.IdTracking);

            // Eliminar las entidades relacionadas.
            cambioEstadoLiquidacionLocal.EstadoAnteriorLiquidacion = null;
            cambioEstadoLiquidacionLocal.EstadoNuevoLiquidacion = null;
            cambioEstadoLiquidacionLocal.Liquidacion = null;
            cambioEstadoLiquidacionLocal.Notificaciones = null;

            switch (entryLocal.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en remoto. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (cambioEstadoLiquidacionRemote != null)
                        this.EntitySyncRepositoryRemote.UpdateCambioEstadoLiquidacion(cambioEstadoLiquidacionLocal);
                    else
                        this.EntitySyncRepositoryRemote.InsertCambioEstadoLiquidacion(cambioEstadoLiquidacionLocal);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (cambioEstadoLiquidacionRemote != null)
                        this.EntitySyncRepositoryRemote.DeleteCambioEstadoLiquidacion(cambioEstadoLiquidacionLocal);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la inserción y actualización de la entrada del tracking.
            if (entryRemote != null)
                this.TrackingRepositoryRemote.UpdateTrackingEntry(entryLocal);
            else
                this.TrackingRepositoryRemote.InsertTrackingEntry(entryLocal);
        }

        private void SyncNotificacionToLocal(TrackingEntry entryRemote)
        {
            // Obtener los objetos de entidad.
            Notificacion notificacionRemote = this.EntitySyncRepositoryRemote.GetNotificacion((Guid)entryRemote.Guid);
            Notificacion notificacionLocal = this.NotificacionRepositoryLocal.GetNotificacion((Guid)entryRemote.Guid);

            // Obtener el objeto de seguimiento local
            TrackingEntry entryLocal = this.TrackingRepositoryLocal.GetTrackingEntry(entryRemote.IdTracking);

            // Eliminar las entidades relacionadas.
            notificacionRemote.CambioEstadoLiquidacion = null;

            switch (entryRemote.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en local. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (notificacionLocal != null)
                    {
                        this.NotificacionRepositoryLocal.Detach(notificacionLocal);
                        this.NotificacionRepositoryLocal.UpdateNotificacion(notificacionRemote);
                    }
                    else
                        this.NotificacionRepositoryLocal.InsertNotificacion(notificacionRemote);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (notificacionLocal != null)
                        this.NotificacionRepositoryLocal.DeleteNotificacion(notificacionRemote);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la entrada del tracking.                    
            if (entryLocal != null)
            {
                this.TrackingRepositoryLocal.Detach(entryLocal);
                this.TrackingRepositoryLocal.UpdateTrackingEntry(entryRemote);
            }
            else
                this.TrackingRepositoryLocal.InsertTrackingEntry(entryRemote);
        }

        private void SyncNotificacionToServer(TrackingEntry entryLocal)
        {
            // Obtener los objetos de entidad.
            Notificacion notificacionRemote = this.EntitySyncRepositoryRemote.GetNotificacion((Guid)entryLocal.Guid);
            Notificacion notificacionLocal = this.NotificacionRepositoryLocal.GetNotificacion((Guid)entryLocal.Guid);

            // Obtener el objeto de seguimiento Remoto
            TrackingEntry entryRemote = this.TrackingRepositoryRemote.GetTrackingEntry(entryLocal.IdTracking);

            // Eliminar las entidades relacionadas.
            notificacionLocal.CambioEstadoLiquidacion = null;

            switch (entryLocal.Operacion)
            {
                // El tratamiento la inserción y actualización es equivalente debido a la correción de errores que puedieran existir.
                case (int)TrackingEntry.OperacionType.Insercion:
                case (int)TrackingEntry.OperacionType.Actualizacion:
                    // Comprobar si la entidad existe en remoto. Puede ocurrir si hubo errores de sincronización o la base de datos se corrompió.                    
                    if (notificacionRemote != null)
                        this.EntitySyncRepositoryRemote.UpdateNotificacion(notificacionLocal);
                    else
                        this.EntitySyncRepositoryRemote.InsertNotificacion(notificacionLocal);
                    break;

                case (int)TrackingEntry.OperacionType.Borrado:
                    if (notificacionRemote != null)
                        this.EntitySyncRepositoryRemote.DeleteNotificacion(notificacionLocal);
                    break;

                default:
                    throw new InvalidOperationException("Operación de sincroniación no esperada.");
            }

            // Idem con la inserción y actualización de la entrada del tracking.
            if (entryRemote != null)
                this.TrackingRepositoryRemote.UpdateTrackingEntry(entryLocal);
            else
                this.TrackingRepositoryRemote.InsertTrackingEntry(entryLocal);
        }

        private IList<TrackingEntry> ListLocalTrackingEntries(DateTime dateFrom)
        {
            using ( TrackingUnitOfWork uow = new TrackingUnitOfWork())
            {
                return uow.ListTrackingEntries(dateFrom);
            }
        }

    }
}
