﻿using LiquidacionGastos.Models;
using LiquidacionGastos.UnitsOfWork;
using LiquidacionGastosService.Contracts;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;


namespace LiquidacionGastosService.Services
{
    public class EntitySyncService : IEntitySyncService
    {
        public Calendario GetCalendario(Guid idCalendario)
        {
            Debug.WriteLine(string.Format("Servidor: Petición de Calendario con ID {0}", idCalendario.ToString()));
            using (CalendarioUnitOfWork uow = new CalendarioUnitOfWork())
            {                
                return uow.GetCalendario(idCalendario);
            }
        }

        public void InsertCalendario(Calendario calendario)
        {
            Debug.WriteLine(string.Format("Servidor: Inserción de Calendario con ID {0}", calendario.IdCalendario.ToString()));
            using (CalendarioUnitOfWork uow = new CalendarioUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.InsertCalendario(calendario);
                uow.Commit();
            }
        }

        public void UpdateCalendario(Calendario calendario)
        {
            Debug.WriteLine(string.Format("Servidor: Actualización de Calendario con ID {0}", calendario.IdCalendario.ToString()));
            using (CalendarioUnitOfWork uow = new CalendarioUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.UpdateCalendario(calendario);
                uow.Commit();
            }
        }

        public void DeleteCalendario(Calendario calendario)
        {
            Debug.WriteLine(string.Format("Servidor: Borrado de Calendario con ID {0}", calendario.IdCalendario.ToString()));
            using (CalendarioUnitOfWork uow = new CalendarioUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.DeleteCalendario(calendario);
                uow.Commit();
            }
        }

        public Departamento GetDepartamento(Guid idDepartamento)
        {
            Debug.WriteLine(string.Format("Servidor: Petición de Departamento con ID {0}", idDepartamento.ToString()));
            using (DepartamentoUnitOfWork uow = new DepartamentoUnitOfWork())
            {
                return uow.GetDepartamento(idDepartamento);
            }
        }

        public void InsertDepartamento(Departamento departamento)
        {
            Debug.WriteLine(string.Format("Servidor: Inserción de Departamento con ID {0}", departamento.IdDepartamento.ToString()));
            using (DepartamentoUnitOfWork uow = new DepartamentoUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.InsertDepartamento(departamento);
                uow.Commit();
            }
        }

        public void UpdateDepartamento(Departamento departamento)
        {
            Debug.WriteLine(string.Format("Servidor: Actualización de Departamento con ID {0}", departamento.IdDepartamento.ToString()));
            using (DepartamentoUnitOfWork uow = new DepartamentoUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.UpdateDepartamento(departamento);
                uow.Commit();
            }
        }

        public void DeleteDepartamento(Departamento departamento)
        {
            Debug.WriteLine(string.Format("Servidor: Borrado de Departamento con ID {0}", departamento.IdDepartamento.ToString()));
            using (DepartamentoUnitOfWork uow = new DepartamentoUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.DeleteDepartamento(departamento);
                uow.Commit();
            }
        }

        public Moneda GetMoneda(int idMoneda)
        {
            Debug.WriteLine(string.Format("Servidor: Petición de Moneda con ID {0}", idMoneda.ToString()));
            using (MonedaUnitOfWork uow = new MonedaUnitOfWork())
            {
                return uow.GetMoneda(idMoneda);
            }
        }

        public void InsertMoneda(Moneda moneda)
        {
            Debug.WriteLine(string.Format("Servidor: Inserción de Moneda con ID {0}", moneda.IdMoneda.ToString()));
            using (MonedaUnitOfWork uow = new MonedaUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.InsertMoneda(moneda);
                uow.Commit();
            }
        }

        public void UpdateMoneda(Moneda moneda)
        {
            Debug.WriteLine(string.Format("Servidor: Actualización de Moneda con ID {0}", moneda.IdMoneda.ToString()));
            using (MonedaUnitOfWork uow = new MonedaUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.UpdateMoneda(moneda);
                uow.Commit();
            }
        }

        public void DeleteMoneda(Moneda moneda)
        {
            Debug.WriteLine(string.Format("Servidor: Borrado de Moneda con ID {0}", moneda.IdMoneda.ToString()));
            using (MonedaUnitOfWork uow = new MonedaUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.DeleteMoneda(moneda);
                uow.Commit();
            }
        }

        public TipoGasto GetTipoGasto(int idTipoGasto)
        {
            Debug.WriteLine(string.Format("Servidor: Petición de TipoGasto con ID {0}", idTipoGasto.ToString()));
            using (TipoGastoUnitOfWork uow = new TipoGastoUnitOfWork())
            {
                return uow.GetTipoGasto(idTipoGasto);
            }
        }

        public void InsertTipoGasto(TipoGasto tipoGasto)
        {
            Debug.WriteLine(string.Format("Servidor: Inserción de TipoGasto con ID {0}", tipoGasto.IdTipoGasto.ToString()));
            using (TipoGastoUnitOfWork uow = new TipoGastoUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.InsertTipoGasto(tipoGasto);
                uow.Commit();
            }
        }

        public void UpdateTipoGasto(TipoGasto tipoGasto)
        {
            Debug.WriteLine(string.Format("Servidor: Actualización de TipoGasto con ID {0}", tipoGasto.IdTipoGasto.ToString()));
            using (TipoGastoUnitOfWork uow = new TipoGastoUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.UpdateTipoGasto(tipoGasto);
                uow.Commit();
            }
        }

        public void DeleteTipoGasto(TipoGasto tipoGasto)
        {
            Debug.WriteLine(string.Format("Servidor: Borrado de TipoGasto con ID {0}", tipoGasto.IdTipoGasto.ToString()));
            using (TipoGastoUnitOfWork uow = new TipoGastoUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.DeleteTipoGasto(tipoGasto);
                uow.Commit();
            }
        }

        public EstadoLiquidacion GetEstadoLiquidacion(int idEstadoLiquidacion)
        {
            Debug.WriteLine(string.Format("Servidor: Petición de EstadoLiquidacion con ID {0}", idEstadoLiquidacion.ToString()));
            using (EstadoLiquidacionUnitOfWork uow = new EstadoLiquidacionUnitOfWork())
            {
                EstadoLiquidacion es = uow.GetEstadoLiquidacion(idEstadoLiquidacion);
                return uow.GetEstadoLiquidacion(idEstadoLiquidacion);
            }
        }

        public void InsertEstadoLiquidacion(EstadoLiquidacion estadoLiquidacion)
        {
            Debug.WriteLine(string.Format("Servidor: Inserción de EstadoLiquidacion con ID {0}", estadoLiquidacion.IdEstadoLiquidacion.ToString()));
            using (EstadoLiquidacionUnitOfWork uow = new EstadoLiquidacionUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.InsertEstadoLiquidacion(estadoLiquidacion);
                uow.Commit();
            }
        }

        public void UpdateEstadoLiquidacion(EstadoLiquidacion estadoLiquidacion)
        {
            Debug.WriteLine(string.Format("Servidor: Actualización de EstadoLiquidacion con ID {0}", estadoLiquidacion.IdEstadoLiquidacion.ToString()));
            using (EstadoLiquidacionUnitOfWork uow = new EstadoLiquidacionUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.UpdateEstadoLiquidacion(estadoLiquidacion);
                uow.Commit();
            }
        }

        public void DeleteEstadoLiquidacion(EstadoLiquidacion estadoLiquidacion)
        {
            Debug.WriteLine(string.Format("Servidor: Borrado de EstadoLiquidacion con ID {0}", estadoLiquidacion.IdEstadoLiquidacion.ToString()));
            using (EstadoLiquidacionUnitOfWork uow = new EstadoLiquidacionUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.DeleteEstadoLiquidacion(estadoLiquidacion);
                uow.Commit();
            }
        }

        public Usuario GetUsuario(Guid idUsuario)
        {
            Debug.WriteLine(string.Format("Servidor: Petición de Usuario con ID {0}", idUsuario.ToString()));
            using (UsuarioUnitOfWork uow = new UsuarioUnitOfWork())
            {
                return uow.GetUsuario(idUsuario);
            }
        }

        public void InsertUsuario(Usuario usuario)
        {
            Debug.WriteLine(string.Format("Servidor: Inserción de Usuario con ID {0}", usuario.IdUsuario.ToString()));
            using (UsuarioUnitOfWork uow = new UsuarioUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.InsertUsuario(usuario);
                uow.Commit();
            }
        }

        public void UpdateUsuario(Usuario usuario)
        {
            Debug.WriteLine(string.Format("Servidor: Actualización de Usuario con ID {0}", usuario.IdUsuario.ToString()));
            using (UsuarioUnitOfWork uow = new UsuarioUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.UpdateUsuario(usuario);
                uow.Commit();
            }
        }

        public void DeleteUsuario(Usuario usuario)
        {
            Debug.WriteLine(string.Format("Servidor: Borrado de Usuario con ID {0}", usuario.IdUsuario.ToString()));
            using (UsuarioUnitOfWork uow = new UsuarioUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.DeleteUsuario(usuario);
                uow.Commit();
            }
        }

        public CambioMoneda GetCambioMoneda(Guid idCambioMoneda)
        {
            Debug.WriteLine(string.Format("Servidor: Petición de CambioMoneda con ID {0}", idCambioMoneda.ToString()));
            using (CambioMonedaUnitOfWork uow = new CambioMonedaUnitOfWork())
            {
                return uow.GetCambioMoneda(idCambioMoneda);
            }
        }

        public void InsertCambioMoneda(CambioMoneda cambioMoneda)
        {
            Debug.WriteLine(string.Format("Servidor: Inserción de CambioMoneda con ID {0}", cambioMoneda.IdCambioMoneda.ToString()));
            using (CambioMonedaUnitOfWork uow = new CambioMonedaUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.InsertCambioMoneda(cambioMoneda);
                uow.Commit();
            }
        }

        public void UpdateCambioMoneda(CambioMoneda cambioMoneda)
        {
            Debug.WriteLine(string.Format("Servidor: Actualización de CambioMoneda con ID {0}", cambioMoneda.IdCambioMoneda.ToString()));
            using (CambioMonedaUnitOfWork uow = new CambioMonedaUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.UpdateCambioMoneda(cambioMoneda);
                uow.Commit();
            }
        }

        public void DeleteCambioMoneda(CambioMoneda cambioMoneda)
        {
            Debug.WriteLine(string.Format("Servidor: Borrado de CambioMoneda con ID {0}", cambioMoneda.IdCambioMoneda.ToString()));
            using (CambioMonedaUnitOfWork uow = new CambioMonedaUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.DeleteCambioMoneda(cambioMoneda);
                uow.Commit();
            }
        }

        public Liquidacion GetLiquidacion(Guid idLiquidacion)
        {
            Debug.WriteLine(string.Format("Servidor: Petición de Liquidacion con ID {0}", idLiquidacion.ToString()));
            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                return uow.GetLiquidacion(idLiquidacion);
            }
        }

        public void InsertLiquidacion(Liquidacion liquidacion)
        {
            Debug.WriteLine(string.Format("Servidor: Inserción de Liquidacion con ID {0}", liquidacion.IdLiquidacion.ToString()));
            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.InsertLiquidacion(liquidacion);
                uow.Commit();
            }
        }

        public void UpdateLiquidacion(Liquidacion liquidacion)
        {
            Debug.WriteLine(string.Format("Servidor: Actualización de Liquidacion con ID {0}", liquidacion.IdLiquidacion.ToString()));
            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.UpdateLiquidacion(liquidacion);
                uow.Commit();
            }
        }

        public void DeleteLiquidacion(Liquidacion liquidacion)
        {
            Debug.WriteLine(string.Format("Servidor: Borrado de Liquidacion con ID {0}", liquidacion.IdLiquidacion.ToString()));
            using (LiquidacionUnitOfWork uow = new LiquidacionUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.DeleteLiquidacion(liquidacion);
                uow.Commit();
            }
        }

        public Gasto GetGasto(Guid idGasto)
        {
            Debug.WriteLine(string.Format("Servidor: Petición de Gasto con ID {0}", idGasto.ToString()));
            using (GastoUnitOfWork uow = new GastoUnitOfWork())
            {
                return uow.GetGasto(idGasto);
            }
        }

        public void InsertGasto(Gasto gasto)
        {
            Debug.WriteLine(string.Format("Servidor: Inserción de Gasto con ID {0}", gasto.IdGasto.ToString()));
            using (GastoUnitOfWork uow = new GastoUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.InsertGasto(gasto);
                uow.Commit();
            }
        }

        public void UpdateGasto(Gasto gasto)
        {
            Debug.WriteLine(string.Format("Servidor: Actualización de Gasto con ID {0}", gasto.IdGasto.ToString()));
            using (GastoUnitOfWork uow = new GastoUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.UpdateGasto(gasto);
                uow.Commit();
            }
        }

        public void DeleteGasto(Gasto gasto)
        {
            Debug.WriteLine(string.Format("Servidor: Borrado de Gasto con ID {0}", gasto.IdGasto.ToString()));
            using (GastoUnitOfWork uow = new GastoUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.DeleteGasto(gasto);
                uow.Commit();
            }
        }

        public CambioEstadoLiquidacion GetCambioEstadoLiquidacion(Guid idCambioEstadoLiquidacion)
        {
            Debug.WriteLine(string.Format("Servidor: Petición de CambioEstadoLiquidacion con ID {0}", idCambioEstadoLiquidacion.ToString()));
            using (CambioEstadoLiquidacionUnitOfWork uow = new CambioEstadoLiquidacionUnitOfWork())
            {
                return uow.GetCambioEstadoLiquidacion(idCambioEstadoLiquidacion);
            }
        }

        public void InsertCambioEstadoLiquidacion(CambioEstadoLiquidacion cambioEstadoLiquidacion)
        {
            Debug.WriteLine(string.Format("Servidor: Inserción de CambioEstadoLiquidacion con ID {0}", cambioEstadoLiquidacion.IdCambioEstadoLiquidacion.ToString()));
            using (CambioEstadoLiquidacionUnitOfWork uow = new CambioEstadoLiquidacionUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.InsertCambioEstadoLiquidacion(cambioEstadoLiquidacion);
                uow.Commit();
            }
        }

        public void UpdateCambioEstadoLiquidacion(CambioEstadoLiquidacion cambioEstadoLiquidacion)
        {
            Debug.WriteLine(string.Format("Servidor: Actualización de CambioEstadoLiquidacion con ID {0}", cambioEstadoLiquidacion.IdCambioEstadoLiquidacion.ToString()));
            using (CambioEstadoLiquidacionUnitOfWork uow = new CambioEstadoLiquidacionUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.UpdateCambioEstadoLiquidacion(cambioEstadoLiquidacion);
                uow.Commit();
            }
        }

        public void DeleteCambioEstadoLiquidacion(CambioEstadoLiquidacion cambioEstadoLiquidacion)
        {
            Debug.WriteLine(string.Format("Servidor: Borrado de CambioEstadoLiquidacion con ID {0}", cambioEstadoLiquidacion.IdCambioEstadoLiquidacion.ToString()));
            using (CambioEstadoLiquidacionUnitOfWork uow = new CambioEstadoLiquidacionUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.DeleteCambioEstadoLiquidacion(cambioEstadoLiquidacion);
                uow.Commit();
            }
        }

        public Notificacion GetNotificacion(Guid idNotificacion)
        {
            Debug.WriteLine(string.Format("Servidor: Petición de Notificacion con ID {0}", idNotificacion.ToString()));
            using (NotificacionUnitOfWork uow = new NotificacionUnitOfWork())
            {
                return uow.GetNotificacion(idNotificacion);
            }
        }

        public void InsertNotificacion(Notificacion notificacion)
        {
            Debug.WriteLine(string.Format("Servidor: Inserción de Notificacion con ID {0}", notificacion.IdNotificacion.ToString()));
            using (NotificacionUnitOfWork uow = new NotificacionUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.InsertNotificacion(notificacion);
                uow.Commit();
            }
        }

        public void UpdateNotificacion(Notificacion notificacion)
        {
            Debug.WriteLine(string.Format("Servidor: Actualización de Notificacion con ID {0}", notificacion.IdNotificacion.ToString()));
            using (NotificacionUnitOfWork uow = new NotificacionUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.UpdateNotificacion(notificacion);
                uow.Commit();
            }
        }

        public void DeleteNotificacion(Notificacion notificacion)
        {
            Debug.WriteLine(string.Format("Servidor: Borrado de Notificacion con ID {0}", notificacion.IdNotificacion.ToString()));
            using (NotificacionUnitOfWork uow = new NotificacionUnitOfWork())
            {
                uow.IsTrackingEnabled = false;
                uow.DeleteNotificacion(notificacion);
                uow.Commit();
            }
        }
    }
}
